Imports System.Runtime.InteropServices
Imports System.Drawing
Imports System.Threading
Imports System.Windows.Forms
Imports Microsoft.Win32.SafeHandles
Imports DirectShowLib

Namespace DatasteadDirectShow

    Public Class DatasteadDirectShowGraph_WithEventsMsg
        Inherits DatasteadDirectShowBaseGraph
        Public m_WinFormHandle As IntPtr = IntPtr.Zero
        Public Overloads Function GetStandardDirectShowInterfaces(ByVal WindowHandle As IntPtr) As Boolean
            Dim Result As Boolean = MyBase.GetStandardDirectShowInterfaces()
            If Result Then
                If WindowHandle <> IntPtr.Zero Then
                    Dim hr As Integer = m_mediaEventEx.SetNotifyWindow(WindowHandle, WM_GRAPHNOTIFY, IntPtr.Zero)
                    DsError.ThrowExceptionForHR(hr)
                    Result = hr = 0
                End If
            End If
            Return Result
        End Function
        Public Shadows Sub HandleGraphEvent()
            Dim CompleteGraphAsynchronously As Boolean = MyBase.HandleGraphEvent()
            If CompleteGraphAsynchronously Then
                RunGraph()
            End If

        End Sub
    End Class
    Public Class DatasteadDirectShowGraph_WithEventsThread
        Inherits DatasteadDirectShowBaseGraph
        Private m_WaitEventCompleted As AutoResetEvent
        Private m_WaitThread As AutoResetEvent
        Private m_EventThread As Thread = Nothing
        Private m_NotificationHandle As SafeWaitHandle = Nothing
        Public Shadows Function GetStandardDirectShowInterfaces() As Boolean
            Dim Result As Boolean = MyBase.GetStandardDirectShowInterfaces()

            If Result Then

                Dim EventHandle As IntPtr
                m_mediaEventEx.GetEventHandle(EventHandle)
                m_NotificationHandle = New SafeWaitHandle(EventHandle, False)
                m_EventThread = New Thread(New ParameterizedThreadStart(AddressOf WaitForEvent))
                m_WaitEventCompleted = New AutoResetEvent(False)
                m_WaitThread = New AutoResetEvent(False)
                m_EventThread.Start(SynchronizationContext.Current)
                m_WaitThread.WaitOne()
            End If
            Return Result
        End Function

        Public Shadows Sub CloseInterfaces()
            If m_mediaEventEx IsNot Nothing Then
                m_WaitEventCompleted.[Set]()
                m_WaitThread.WaitOne()
            End If
            MyBase.CloseInterfaces()
        End Sub

        Public Sub WaitForEvent(ByVal context__1 As Object)
            m_WaitThread.[Set]()
            Dim Context__2 As SynchronizationContext = DirectCast(context__1, SynchronizationContext)
            Dim NotificationEvent As New AutoResetEvent(True)
            NotificationEvent.SafeWaitHandle = m_NotificationHandle
            'm_WaitEventCompleted

            Dim waitHandleList As WaitHandle() = New WaitHandle() {NotificationEvent, m_WaitEventCompleted}

            Dim Result As Integer
            Do
                Result = WaitHandle.WaitAny(waitHandleList, Timeout.Infinite, False)
                If Result = 0 Then
                    HandleGraphEventFromThread(Context__2)
                End If
            Loop While Result <> 1
            m_WaitThread.[Set]()
        End Sub
        Private Sub HandleGraphEventFromThread(ByVal Context As SynchronizationContext)
            Dim CompleteGraphAsynchronously As Boolean = MyBase.HandleGraphEvent()
            If CompleteGraphAsynchronously AndAlso m_GraphCanRun Then

                Context.Post(New SendOrPostCallback(AddressOf RunGraph), Nothing)
            End If
        End Sub


    End Class


    Public Class DatasteadDirectShowBaseGraph
        Inherits Datastead.RTSPFilter
        Implements IDisposable
        Public Delegate Sub GraphRunResultCallback(ByVal Success As Boolean)
        Public GraphRunResult As GraphRunResultCallback = Nothing

        Public Const WM_GRAPHNOTIFY As Integer = &H8000 + 1

        Private disposedValue As Boolean = False        ' To detect redundant calls

        ' IDisposable
        Protected Overridable Sub Dispose(ByVal disposing As Boolean)
            If Not Me.disposedValue Then
                If disposing Then
                    ' TODO: free other state (managed objects).
                End If
                CloseInterfaces()
                ' TODO: free your own state (unmanaged objects).
                ' TODO: set large fields to null.
            End If
            Me.disposedValue = True
        End Sub

        Public Sub Dispose() Implements IDisposable.Dispose
            ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(True)
            GC.SuppressFinalize(Me)
        End Sub

        ' Application-defined message to notify app of filtergraph events
        Public Enum VideoRendererToUse
            vr_None
            vr_Default
            vr_DatasteadRenderer
            vr_StandardRenderer
            vr_VMR7
            vr_VMR9
            vr_NullRenderer
        End Enum

        Private m_lock As New Object()
        Protected m_GraphCanRun As Boolean = True

        Public m_filtergraph As FilterGraph = Nothing
        Public m_mediaControl As IMediaControl = Nothing
        Public m_mediaEventEx As IMediaEventEx = Nothing
        Public m_graphBuilder As IGraphBuilder = Nothing
        Public m_DatasteadRTSPSource As IBaseFilter = Nothing
        Public m_DatasteadRTSPSourceConfig As IDatasteadRTSPSourceConfig = Nothing
        Public m_DatasteadRTSPConfigHelper As DatasteadRTSPConfigHelper = Nothing
        Public m_DatasteadRTSPSampleCallback As IDatasteadRTSPSampleCallback = Nothing
        Public m_captureGraphBuilder As ICaptureGraphBuilder2 = Nothing
        Private m_videoWindow As IVideoWindow = Nothing
        Private m_VMRWindowlessControl7 As IVMRWindowlessControl = Nothing
        Private m_VMRWindowlessControl9 As IVMRWindowlessControl9 = Nothing
        Public m_rot As DsROTEntry = Nothing

        Public m_VideoStreamEnabled As Boolean = True
        Public m_AudioStreamEnabled As Boolean = True
        Public m_VideoStreamVisible As Boolean = True
        Public m_AudioStreamRendered As Boolean = True

        Public Delegate Sub GraphEvent(ByVal EventCode As Integer, ByVal Param1 As Integer, ByVal Param2 As Integer)
        Public m_GraphEventCallback As GraphEvent = Nothing

        Public Delegate Sub AddToLog(ByVal LogString As String)
        Public m_AddToLog As AddToLog = Nothing

        Public m_DisplayWindow As Control = Nothing
        Public m_PictureFromMemoryBitmap As PictureBox = Nothing

        Public m_VideoRendererToUse As VideoRendererToUse = VideoRendererToUse.vr_Default

        Public ReadOnly Property Lock() As Object
            Get
                Return m_lock
            End Get
        End Property

        Public Property GraphCanRun() As Boolean
            Get
                Return m_GraphCanRun
            End Get
            Set(ByVal value As Boolean)
                m_GraphCanRun = value
            End Set
        End Property

        Public Function GetAudioDevices() As String
            SyncLock m_lock
                Dim DatasteadRTSPSource As IBaseFilter = DirectCast(Activator.CreateInstance(Type.GetTypeFromCLSID(Datastead.RTSPFilter.DatasteadRtspFilterCLSID)), IBaseFilter)
                Dim DatasteadRTSPSourceConfig As IDatasteadRTSPSourceConfig = DirectCast(DatasteadRTSPSource, IDatasteadRTSPSourceConfig)
                Dim RTSPConfigHelper As New DatasteadRTSPConfigHelper(DatasteadRTSPSourceConfig)
                Dim AudioDevices As IntPtr
                Dim Result As String = ""
                If RTSPConfigHelper.GetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_GetAudioDevices_str, AudioDevices) = 0 Then
                    Result = Marshal.PtrToStringAuto(AudioDevices)
                End If
                Marshal.ReleaseComObject(DatasteadRTSPSourceConfig)
                Marshal.ReleaseComObject(DatasteadRTSPSource)
                Return Result
            End SyncLock
        End Function

        Public Function GetAudioDevicesAsItems() As String()
            SyncLock m_lock
                Dim AudioDevicesGot As String = GetAudioDevices()
                Dim AudioDevices As String() = AudioDevicesGot.Split(ControlChars.Lf)
                Return AudioDevices
            End SyncLock
        End Function

        Public Function RenderVideoPin() As Boolean
            SyncLock m_lock
                Dim Result As Boolean = False
                Dim pVideoOutPin As IPin
                If m_captureGraphBuilder.FindPin(m_DatasteadRTSPSource, PinDirection.Output, Nothing, MediaType.Video, True, 0, _
                 pVideoOutPin) = 0 Then
                    If m_VideoRendererToUse = VideoRendererToUse.vr_Default Then
                        If m_graphBuilder.Render(pVideoOutPin) = 0 Then

                            m_videoWindow = DirectCast(m_graphBuilder, IVideoWindow)

                            ' Set the video window to be a child of the main window
                            Dim hr As Integer = m_videoWindow.put_Owner(m_DisplayWindow.Handle)
                            DsError.ThrowExceptionForHR(hr)

                            hr = m_videoWindow.put_WindowStyle(WindowStyle.Child Or WindowStyle.ClipChildren Or WindowStyle.ClipSiblings)
                            DsError.ThrowExceptionForHR(hr)

                            ' Use helper function to position video window in client rect 
                            ' of main application window
                            ResizeVideoWindow()

                            ' Make the video window visible, now that it is properly positioned
                            hr = m_videoWindow.put_Visible(OABool.[True])
                            DsError.ThrowExceptionForHR(hr)

                            Result = True
                        End If
                    ElseIf (m_VideoRendererToUse = VideoRendererToUse.vr_DatasteadRenderer) OrElse (m_VideoRendererToUse = VideoRendererToUse.vr_StandardRenderer) Then
                        Dim Added As Boolean = False

                        Dim RendererFilter As IBaseFilter = Nothing

                        If m_VideoRendererToUse = VideoRendererToUse.vr_DatasteadRenderer Then
                            RendererFilter = DirectCast(Activator.CreateInstance(Type.GetTypeFromCLSID(DatasteadVideoRendererCLSID)), IBaseFilter)
                            If RendererFilter IsNot Nothing Then
                                Added = m_graphBuilder.AddFilter(RendererFilter, "Datastead Video Renderer") = 0
                            End If
                        ElseIf m_VideoRendererToUse = VideoRendererToUse.vr_StandardRenderer Then
                            If RendererFilter IsNot Nothing Then
                                RendererFilter = DirectCast(New VideoRenderer(), IBaseFilter)
                                Added = m_graphBuilder.AddFilter(RendererFilter, "Video Renderer") = 0
                            End If
                        End If

                        If Added Then
                            If m_captureGraphBuilder.RenderStream(Nothing, Nothing, m_DatasteadRTSPSource, Nothing, RendererFilter) = 0 Then
                                m_videoWindow = DirectCast(m_graphBuilder, IVideoWindow)

                                ' Set the video window to be a child of the main window
                                Dim hr As Integer = m_videoWindow.put_Owner(m_DisplayWindow.Handle)
                                DsError.ThrowExceptionForHR(hr)

                                hr = m_videoWindow.put_WindowStyle(WindowStyle.Child Or WindowStyle.ClipChildren Or WindowStyle.ClipSiblings)
                                DsError.ThrowExceptionForHR(hr)

                                ' Use helper function to position video window in client rect 
                                ' of main application window
                                ResizeVideoWindow()

                                ' Make the video window visible, now that it is properly positioned
                                hr = m_videoWindow.put_Visible(OABool.[True])
                                DsError.ThrowExceptionForHR(hr)

                                Result = True
                            End If
                        End If
                        If RendererFilter IsNot Nothing Then
                            Marshal.ReleaseComObject(RendererFilter)
                        End If
                    ElseIf m_VideoRendererToUse = VideoRendererToUse.vr_VMR7 Then
                        Dim vmr7 As New VideoMixingRenderer()
                        If m_graphBuilder.AddFilter(DirectCast(vmr7, IBaseFilter), "Video Mixing Renderer 7") = 0 Then
                            Dim vmr7Config As IVMRFilterConfig = DirectCast(vmr7, IVMRFilterConfig)
                            vmr7Config.SetRenderingMode(VMRMode.Windowless)

                            m_VMRWindowlessControl7 = DirectCast(vmr7, IVMRWindowlessControl)
                            m_VMRWindowlessControl7.SetVideoClippingWindow(m_DisplayWindow.Handle)
                            m_VMRWindowlessControl7.SetAspectRatioMode(VMRAspectRatioMode.None)
                            If m_captureGraphBuilder.RenderStream(Nothing, MediaType.Video, m_DatasteadRTSPSource, Nothing, DirectCast(vmr7, IBaseFilter)) = 0 Then
                                ResizeVideoWindow()
                            End If
                            Marshal.ReleaseComObject(vmr7Config)
                            Result = True
                        End If
                        Marshal.ReleaseComObject(vmr7)
                    ElseIf m_VideoRendererToUse = VideoRendererToUse.vr_VMR9 Then
                        Dim vmr9 As New VideoMixingRenderer9()
                        If m_graphBuilder.AddFilter(DirectCast(vmr9, IBaseFilter), "Video Mixing Renderer 9") = 0 Then
                            Dim vmr9Config As IVMRFilterConfig9 = DirectCast(vmr9, IVMRFilterConfig9)
                            vmr9Config.SetRenderingMode(VMR9Mode.Windowless)
                            vmr9Config.SetNumberOfStreams(1)

                            m_VMRWindowlessControl9 = DirectCast(vmr9, IVMRWindowlessControl9)
                            m_VMRWindowlessControl9.SetVideoClippingWindow(m_DisplayWindow.Handle)
                            m_VMRWindowlessControl9.SetAspectRatioMode(VMR9AspectRatioMode.None)
                            If m_captureGraphBuilder.RenderStream(Nothing, MediaType.Video, m_DatasteadRTSPSource, Nothing, DirectCast(vmr9, IBaseFilter)) = 0 Then
                                ResizeVideoWindow()
                            End If
                            Marshal.ReleaseComObject(vmr9Config)
                            Result = True
                        End If
                        Marshal.ReleaseComObject(vmr9)
                    ElseIf m_VideoRendererToUse = VideoRendererToUse.vr_NullRenderer Then
                        Dim Null_Renderer As New NullRenderer()
                        If m_graphBuilder.AddFilter(DirectCast(Null_Renderer, IBaseFilter), "Null Renderer") = 0 Then
                            ' ok
                            If m_captureGraphBuilder.RenderStream(Nothing, MediaType.Video, m_DatasteadRTSPSource, Nothing, DirectCast(Null_Renderer, IBaseFilter)) = 0 Then
                            End If
                            Result = True
                        End If
                        Marshal.ReleaseComObject(Null_Renderer)
                    End If
                End If
                Return Result
            End SyncLock
        End Function

        Public Function RenderAudioPin() As Boolean
            SyncLock m_lock
                Dim Result As Boolean = False
                Dim pPin As IPin
                If m_captureGraphBuilder.FindPin(m_DatasteadRTSPSource, PinDirection.Output, Nothing, MediaType.Audio, True, 0, _
                 pPin) = 0 Then
                    If m_graphBuilder.Render(pPin) = 0 Then
                        Result = True
                    End If
                    pPin = Nothing
                End If
                Return Result
            End SyncLock
        End Function

        Protected Function GetStandardDirectShowInterfaces() As Boolean
            Dim Result As Boolean = False
            SyncLock m_lock
                If m_filtergraph Is Nothing Then
                    m_filtergraph = New FilterGraph()
                    If m_filtergraph IsNot Nothing Then
                        m_graphBuilder = DirectCast(m_filtergraph, IGraphBuilder)
                        m_mediaControl = DirectCast(m_graphBuilder, IMediaControl)
                        m_mediaEventEx = DirectCast(m_graphBuilder, IMediaEventEx)

                        m_captureGraphBuilder = DirectCast(New CaptureGraphBuilder2(), ICaptureGraphBuilder2)
                        Result = m_captureGraphBuilder.SetFiltergraph(m_graphBuilder) = 0
                    End If
                End If
            End SyncLock
            Return Result
        End Function

        Public Function AddToROT() As Boolean
            Dim Result As Boolean = False
            If m_graphBuilder IsNot Nothing Then
                SyncLock m_lock
                    ' Add our graph to the running object table, which will allow
                    ' the GraphEdit application to "spy" on our graph
                    m_rot = New DsROTEntry(m_graphBuilder)
                    Result = m_rot IsNot Nothing
                End SyncLock
            End If
            Return Result
        End Function

        Public Function AddDatasteadFilterAndQueryFilterInterfaces() As Boolean
            Dim FilterAdded As Boolean = False
            SyncLock m_lock
                m_DatasteadRTSPSource = DirectCast(Activator.CreateInstance(Type.GetTypeFromCLSID(Datastead.RTSPFilter.DatasteadRtspFilterCLSID)), IBaseFilter)
                If m_DatasteadRTSPSource IsNot Nothing Then
                    If m_graphBuilder.AddFilter(m_DatasteadRTSPSource, Datastead.RTSPFilter.FilterName) = 0 Then
                        ' if ok
                        m_DatasteadRTSPSourceConfig = DirectCast(m_DatasteadRTSPSource, IDatasteadRTSPSourceConfig)
                        m_DatasteadRTSPConfigHelper = New DatasteadRTSPConfigHelper(m_DatasteadRTSPSourceConfig)
                        m_DatasteadRTSPSampleCallback = DirectCast(m_DatasteadRTSPSource, IDatasteadRTSPSampleCallback)
                        FilterAdded = True
                    End If
                Else
                    MessageBox.Show(Datastead.RTSPFilter.FilterName + "not registered! This demo is built with ""Any"" as target platform, so it will run the x86 filter on a 32bit OS and the x64 filter on a 64bit OS, verify that you have registered the corresponding filter.")
                End If
            End SyncLock
            Return FilterAdded
        End Function

        Public Sub CloseInterfaces()
            ' invoked also in the Form1.Designer.cs -> dispose
            SyncLock m_lock
                Dim CanClose As Boolean = True

                If m_DatasteadRTSPSourceConfig IsNot Nothing Then
                    Dim CurrentState As Integer
                    m_DatasteadRTSPSourceConfig.GetInt(RTSPConfigParam.RTSP_Source_GetState_int.ToString(), CurrentState)
                    If DirectCast(CurrentState, TSourceState) = TSourceState.state_connecting_async Then
                        CanClose = False
                        ''' first we cancel the connection, then the graph will be closed by the event callback
                        m_DatasteadRTSPSourceConfig.Action(RTSPConfigParam.RTSP_Action_CancelPendingConnection.ToString(), "")
                    End If
                End If

                If CanClose Then

                    If m_mediaControl IsNot Nothing Then
                        m_mediaControl.[Stop]()
                    End If

                    ' Relinquish ownership (IMPORTANT!) of the video window.
                    ' Failing to call put_Owner can lead to assert failures within
                    ' the video renderer, as it still assumes that it has a valid
                    ' parent window.
                    If m_videoWindow IsNot Nothing Then
                        m_videoWindow.put_Visible(OABool.[False])
                        m_videoWindow.put_Owner(IntPtr.Zero)
                    End If

                    If m_DatasteadRTSPSourceConfig IsNot Nothing Then
                        m_DatasteadRTSPSourceConfig.Action(RTSPConfigParam.RTSP_Action_CancelPendingConnection.ToString(), "")
                    End If

                    ' Remove filter graph from the running object table
                    If m_rot IsNot Nothing Then
                        m_rot.Dispose()
                        m_rot = Nothing
                    End If

                    ' Release DirectShow interfaces
                    If m_DatasteadRTSPSourceConfig IsNot Nothing Then
                        Marshal.ReleaseComObject(m_DatasteadRTSPSourceConfig)
                        m_DatasteadRTSPSourceConfig = Nothing
                    End If
                    If m_DatasteadRTSPSampleCallback IsNot Nothing Then
                        Marshal.ReleaseComObject(m_DatasteadRTSPSampleCallback)
                        m_DatasteadRTSPSampleCallback = Nothing
                    End If
                    If m_DatasteadRTSPSource IsNot Nothing Then
                        Marshal.ReleaseComObject(m_DatasteadRTSPSource)
                        m_DatasteadRTSPSource = Nothing
                    End If
                    If m_mediaControl IsNot Nothing Then
                        Marshal.ReleaseComObject(m_mediaControl)
                        m_mediaControl = Nothing
                    End If
                    If m_mediaEventEx IsNot Nothing Then
                        Marshal.ReleaseComObject(m_mediaEventEx)
                        m_mediaEventEx = Nothing
                    End If
                    If m_videoWindow IsNot Nothing Then
                        Marshal.ReleaseComObject(m_videoWindow)
                        m_videoWindow = Nothing
                    End If
                    If m_VMRWindowlessControl7 IsNot Nothing Then
                        Marshal.ReleaseComObject(m_VMRWindowlessControl7)
                        m_VMRWindowlessControl7 = Nothing
                    End If
                    If m_VMRWindowlessControl9 IsNot Nothing Then
                        Marshal.ReleaseComObject(m_VMRWindowlessControl9)
                        m_VMRWindowlessControl9 = Nothing
                    End If
                    If m_graphBuilder IsNot Nothing Then
                        Marshal.ReleaseComObject(m_graphBuilder)
                        m_graphBuilder = Nothing
                    End If
                    If m_captureGraphBuilder IsNot Nothing Then
                        Marshal.ReleaseComObject(m_captureGraphBuilder)
                        m_captureGraphBuilder = Nothing
                    End If
                    If m_filtergraph IsNot Nothing Then
                        Marshal.ReleaseComObject(m_filtergraph)
                        m_filtergraph = Nothing
                    End If
                    GC.Collect()
                End If
            End SyncLock
        End Sub

        Public Function HandleGraphEvent() As Boolean
            Dim hr As Integer = 0
            Dim evCode As EventCode
            Dim evParam1 As IntPtr, evParam2 As IntPtr
            Dim CompleteGraphAsynchronously As Boolean = False
            Dim CanCloseInterfaces As Boolean = False

            SyncLock m_lock
                If m_mediaEventEx.GetEvent(evCode, evParam1, evParam2, 0) = 0 Then
                    If m_mediaControl IsNot Nothing Then
                        If CInt(evCode) = EC_RTSPNOTIFY Then
                            Select Case evParam1.ToInt32()
                                Case EC_RTSP_PARAM1_OPENURLASYNC_CONNECTION_RESULT
                                    If True Then
                                        If evParam2 <> IntPtr.Zero Then
                                            ''' we have received the notification that the URL is now connected, we can render the pins and run the graph.
                                            CompleteGraphAsynchronously = True
                                        Else
                                            m_AddToLog("error: asynchronous URL connection failed, check if the URL is correct or the server is alive")
                                            CanCloseInterfaces = True
                                        End If

                                    End If
                                    Exit Select
                                Case EC_RTSP_PARAM1_DEVICELOST_RECONNECTING
                                    If True Then
                                        m_AddToLog("reconnecting...")
                                    End If
                                    Exit Select
                                Case EC_RTSP_PARAM1_DEVICELOST_RECONNECTED
                                    If True Then
                                        m_AddToLog("reconnected.")
                                    End If
                                    Exit Select
                                Case EC_RTSP_PARAM1_RECORDTONEWFILE_COMPLETED
                                    If True Then
                                        Dim LastRecordingFilename As IntPtr
                                        m_DatasteadRTSPConfigHelper.GetStr(RTSPConfigParam.RTSP_LastRecorded_FileName_str, LastRecordingFilename)
                                        m_AddToLog("recording to " + Marshal.PtrToStringUni(LastRecordingFilename) + " completed: ")

                                        Dim RecordedFileSizeKb As Integer
                                        m_DatasteadRTSPConfigHelper.GetInt(RTSPConfigParam.RTSP_LastRecorded_FileSizeKb_int, RecordedFileSizeKb)
                                        m_AddToLog("file size in Kb: " + RecordedFileSizeKb.ToString())

                                        Dim RecordedDurationMs As Integer
                                        m_DatasteadRTSPConfigHelper.GetInt(RTSPConfigParam.RTSP_LastRecorded_ClipDurationMs_int, RecordedDurationMs)
                                        m_AddToLog("duration in ms: " + RecordedDurationMs.ToString())

                                        Dim FrameCount As Integer
                                        m_DatasteadRTSPConfigHelper.GetInt(RTSPConfigParam.RTSP_LastRecorded_VideoFrameCount_int, FrameCount)
                                        m_AddToLog("frame count: " + FrameCount.ToString())

                                        Dim NewRecordingFilename As IntPtr
                                        m_DatasteadRTSPConfigHelper.GetStr(RTSPConfigParam.RTSP_CurrentRecording_FileName_str, NewRecordingFilename)
                                        m_AddToLog("now recording to: " + Marshal.PtrToStringUni(NewRecordingFilename))
                                    End If
                                    Exit Select
                                Case EC_RTSP_PARAM1_FRAME_CAPTURE_SUCCEEDED
                                    If True Then
                                        If evParam2 = IntPtr.Zero Then
                                            ' in this case this is a frame capture to file
                                            m_AddToLog("frame capture to file successful.")
                                        Else
                                            m_AddToLog("frame capture to memory bitmap successful.")
                                            If m_PictureFromMemoryBitmap IsNot Nothing Then
                                                m_PictureFromMemoryBitmap.Image = Image.FromHbitmap(evParam2)
                                            End If
                                        End If
                                    End If
                                    Exit Select
                                Case EC_RTSP_PARAM1_FRAME_CAPTURE_FAILED
                                    If True Then
                                        m_AddToLog("frame capture failed, GetLastError return code: " + evParam2.ToString())
                                    End If
                                    Exit Select
                                Case EC_RTSP_PARAM1_ONVIF_SNAPSHOT_SUCCEEDED
                                    If True Then
                                        m_AddToLog("ONVIF snapshot succeeded")
                                    End If
                                    Exit Select
                                Case EC_RTSP_PARAM1_ONVIF_SNAPSHOT_FAILED
                                    If True Then
                                        m_AddToLog("ONVIF snapshot failed")
                                    End If
                                    Exit Select

                            End Select
                        ElseIf evCode = EventCode.DeviceLost Then
                            m_AddToLog("device lost!")

                        End If
                    End If
                    If m_GraphEventCallback IsNot Nothing Then
                        m_GraphEventCallback(CInt(evCode), evParam1.ToInt32(), evParam2.ToInt32())
                    End If
                    ' Free event parameters to prevent memory leaks associated with
                    ' event parameter data.  While this application is not interested
                    ' in the received events, applications should always process them.
                    hr = m_mediaEventEx.FreeEventParams(evCode, evParam1, evParam2)

                    DsError.ThrowExceptionForHR(hr)
                End If
            End SyncLock

            If CanCloseInterfaces Then
                CloseInterfaces()
            End If

            Return CompleteGraphAsynchronously
        End Function
        Public Function RunGraph() As Boolean
            SyncLock m_lock
                Dim hr As Integer = 0

                Dim [Step] As String = ""

                Dim StreamInfo As IntPtr
                hr = m_DatasteadRTSPConfigHelper.GetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_StreamInfo_str, StreamInfo)
                If hr = 0 Then
                    ' if success
                    m_AddToLog("Connected." & vbCr & vbLf & vbCr & vbLf + Marshal.PtrToStringUni(StreamInfo))
                Else
                    DsError.ThrowExceptionForHR(hr)
                End If

                ' END DATASTEAD RTSP INITIALIZATION CODE 
                ' ---------------------------------------------------------

                If m_VideoStreamVisible Then
                    RenderVideoPin()
                End If

                If m_AudioStreamRendered Then
                    RenderAudioPin()
                End If


                ' Start previewing video data
                [Step] = "run graph"

                hr = m_mediaControl.Run()
                ' when Run() succeeds it may also return 1 (S_FALSE), that means that not all the filters have yet completed the transition. This is normal.
                ' ok
                If hr >= 0 Then
                Else
                    m_AddToLog((Convert.ToString("error: CompleteGraph() failed at step """) & [Step]) + """")
                End If

                GraphRunResult(hr >= 0)

                Return hr >= 0
            End SyncLock
        End Function

        Public Sub ResizeVideoWindow()
            SyncLock m_lock
                If m_DisplayWindow Is Nothing Then
                    Return
                End If
                If m_VMRWindowlessControl9 IsNot Nothing Then
                    m_VMRWindowlessControl9.SetVideoPosition(Nothing, DsRect.FromRectangle(m_DisplayWindow.ClientRectangle))
                ElseIf m_VMRWindowlessControl7 IsNot Nothing Then
                    m_VMRWindowlessControl7.SetVideoPosition(Nothing, DsRect.FromRectangle(m_DisplayWindow.ClientRectangle))
                ElseIf m_videoWindow IsNot Nothing Then
                    m_videoWindow.SetWindowPosition(0, 0, m_DisplayWindow.ClientSize.Width, m_DisplayWindow.ClientSize.Height)
                End If
            End SyncLock
        End Sub

        Public Class DatasteadRTSPConfigHelper
            Private m_DatasteadRTSPSourceConfig As IDatasteadRTSPSourceConfig

            Public Sub New(ByVal DatasteadRTSPSourceConfig As IDatasteadRTSPSourceConfig)
                m_DatasteadRTSPSourceConfig = DatasteadRTSPSourceConfig
            End Sub

            Public Function GetBool(ByVal ParamID As RTSPConfigParam, ByRef Value As Boolean) As Integer
                Return m_DatasteadRTSPSourceConfig.GetBool(ParamID.ToString(), Value)
            End Function

            Public Function GetDouble(ByVal ParamID As RTSPConfigParam, ByRef Value As Double) As Integer
                Return m_DatasteadRTSPSourceConfig.GetDouble(ParamID.ToString(), Value)
            End Function

            Public Function GetInt(ByVal ParamID As RTSPConfigParam, ByRef Value As Integer) As Integer
                Return m_DatasteadRTSPSourceConfig.GetInt(ParamID.ToString(), Value)
            End Function

            Public Function GetStr(ByVal ParamID As RTSPConfigParam, ByRef Value As IntPtr) As Integer
                Return m_DatasteadRTSPSourceConfig.GetStr(ParamID.ToString(), Value)
            End Function

            Public Function SetBool(ByVal ParamID As RTSPConfigParam, ByVal Value As Boolean) As Integer
                Return m_DatasteadRTSPSourceConfig.SetBool(ParamID.ToString(), Value)
            End Function

            Public Function SetDouble(ByVal ParamID As RTSPConfigParam, ByVal Value As Double) As Integer
                Return m_DatasteadRTSPSourceConfig.SetDouble(ParamID.ToString(), Value)
            End Function

            Public Function SetInt(ByVal ParamID As RTSPConfigParam, ByVal Value As Integer) As Integer
                Return m_DatasteadRTSPSourceConfig.SetInt(ParamID.ToString(), Value)
            End Function

            Public Function SetStr(ByVal ParamID As RTSPConfigParam, ByVal Value As String) As Integer
                Return m_DatasteadRTSPSourceConfig.SetStr(ParamID.ToString(), Value)
            End Function

            Public Function Action(ByVal ActionID As RTSPConfigParam, ByVal [Option] As String) As Integer
                Return m_DatasteadRTSPSourceConfig.Action(ActionID.ToString(), [Option])
            End Function
        End Class

    End Class
End Namespace
