'using System.Linq;
Imports System.Runtime.InteropServices
Imports System.ComponentModel
'using System.Collections.Generic;

Namespace Datastead
    ''' Datastead RTSP/RTMP DirectShow source filter.
    Public Class RTSPFilter
        Public Const FilterName As String = "Datastead RTSP/RTMP/HTTP/ONVIF DirectShow Source Filter"
        ' values to pass to SetInt (RTSP_Source_RTSPTransport_int, value)
        Public Const RTSPTransportMode_Auto As Integer = 0
        Public Const RTSPTransportMode_TCP As Integer = 1
        Public Const RTSPTransportMode_UDP As Integer = 2
        Public Const RTSPTransportMode_HTTP As Integer = 3
        Public Const RTSPTransportMode_Udp_Multicast As Integer = 4


        Const EC_USER As Integer = &H8000
        ' from DirectShow, comment this line if you get an "already defined" error
        Public Const EC_RTSPNOTIFY As Integer = EC_USER + &H4000
        Public Const EC_RTSP_PARAM1_DISKFULL As Integer = 1
        Public Const EC_RTSP_PARAM1_DEVICELOST_RECONNECTING As Integer = 3
        Public Const EC_RTSP_PARAM1_DEVICELOST_RECONNECTED As Integer = 4
        Public Const EC_RTSP_PARAM1_RECORDTONEWFILE_COMPLETED As Integer = 5
        Public Const EC_RTSP_PARAM1_OPENURLASYNC_CONNECTION_RESULT As Integer = 6
        Public Const EC_RTSP_PARAM1_FRAME_CAPTURE_SUCCEEDED As Integer = 7
        Public Const EC_RTSP_PARAM1_FRAME_CAPTURE_FAILED As Integer = 8
        Public Const EC_RTSP_PARAM1_ONVIF_SNAPSHOT_SUCCEEDED As Integer = 9
        Public Const EC_RTSP_PARAM1_ONVIF_SNAPSHOT_FAILED As Integer = 10

        Public Shared ReadOnly DatasteadRtspFilterCLSID As New Guid("55D1139D-5E0D-4123-9AED-575D7B039569")

        Public Shared ReadOnly DatasteadVideoRendererCLSID As New Guid("C7CC1A23-8B8A-4BFD-A96C-B5E735E055BA")

        <ComImport(), Guid("D7557B82-3FA4-4F4F-B7CF-96108202E4AF"), System.Security.SuppressUnmanagedCodeSecurity(), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
        Public Interface IDatasteadRTSPSourceConfig
            <PreserveSig()> _
            Function GetBool(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <Out()> ByRef Value As Boolean) As Integer
            <PreserveSig()> _
            Function GetDouble(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <Out()> ByRef Value As Double) As Integer
            <PreserveSig()> _
            Function GetInt(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <Out()> ByRef Value As Integer) As Integer
            <PreserveSig()> _
            Function GetStr(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <Out()> ByRef Value As IntPtr) As Integer
            <PreserveSig()> _
            Function SetBool(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <[In]()> ByVal Value As Boolean) As Integer
            <PreserveSig()> _
            Function SetDouble(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <[In]()> ByVal Value As Double) As Integer
            <PreserveSig()> _
            Function SetInt(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <[In]()> ByVal Value As Integer) As Integer
            <PreserveSig()> _
            Function SetStr(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal Value As String) As Integer
            <PreserveSig()> _
            Function Action(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ActionID As String, <[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal [Option] As String) As Integer
        End Interface

        <ComImport(), Guid("932C369A-3335-45BB-B5F4-A6CF77F9B40C"), System.Security.SuppressUnmanagedCodeSecurity(), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
        Public Interface IDatasteadRTSPSourceConfig2
            Inherits IDatasteadRTSPSourceConfig
            <PreserveSig()> _
            Function GetIntPtr(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <Out()> ByRef Value As IntPtr) As Integer
        End Interface

        <ComImport(), Guid("6FCA28BE-7F96-467A-BB08-0611FC8162B7"), System.Security.SuppressUnmanagedCodeSecurity(), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
        Public Interface IDatasteadRTSPSourceConfig3
            Inherits IDatasteadRTSPSourceConfig
            <PreserveSig()> _
            Function GetInt64(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <Out()> ByRef Value As Int64) As Integer
            <PreserveSig()> _
            Function SetInt64(<[In](), MarshalAs(UnmanagedType.LPWStr)> ByVal ParamID As String, <[In]()> ByVal Value As Int64) As Integer
        End Interface

        '_________________________________________________________________________________
        ' Current state: returned by DatasteadRTSPSourceConfig.GetInt (RTSP_Source_GetState_int, out SourceState)
        '_________________________________________________________________________________

        Public Enum TSourceState
            state_disconnected
            state_connecting_async
            state_connecting_sync
            state_reconnecting
            state_connected
            state_previewing
            state_recording_paused
            state_recording_active
        End Enum

        '_________________________________________________________________________________
        ' RTSP Filter configuration identifiers that are supported as ParamID by the IDatasteadRTSPSourceConfig functions
        '_________________________________________________________________________________
        Public Enum RTSPConfigParam
            RTSP_Action_OpenURL
            RTSP_Action_OpenURLAsync
            RTSP_Action_GetONVIFSnapshot
            RTSP_Action_GetONVIFSnapshotAsync
            RTSP_Action_RecordToNewFileNow
            RTSP_Action_CancelPendingConnection
            RTSP_Action_PauseURL
            RTSP_Action_ResumeURL
            RTSP_Action_PauseRecording
            RTSP_Action_ResumeRecording
            RTSP_Action_CaptureFrame
            RTSP_Action_UpdateDuration

            RTSP_Source_RecordingFileName_str
            RTSP_Source_RecordingBacktimedStartSeconds_int
            RTSP_Source_Recording_Title_str
            RTSP_Source_PlayableWhileRecording_int
            RTSP_Source_ContinuousRecording_bool
            RTSP_Source_MaxAnalyzeDuration_int
            RTSP_Source_AutoReconnect_bool
            RTSP_Source_NoTranscoding_bool
            RTSP_Source_DeviceLostTimeOut_int
            RTSP_Source_BufferDuration_int
            RTSP_Source_LowDelay_int
            RTSP_Source_SampleDeliveryMode_int
            RTSP_Source_ConnectionTimeOut_int
            RTSP_Source_RTSPTransport_int
            RTSP_Source_RTSPRange_str
            RTSP_Source_Format_str
            RTSP_Source_FrameRate_double
            RTSP_Source_AverageTimePerFrame100ns_int
            RTSP_Source_DurationMs_int
            RTSP_Source_Duration100ns_int64
            RTSP_Source_VideoSize_str
            RTSP_Source_VideoCodec_str
            RTSP_Source_AuthUser_str
            RTSP_Source_AuthPassword_str
            RTSP_Source_StreamInfo_str
            RTSP_Source_StartTime_int
            RTSP_Source_Threads_int
            RTSP_Source_IsURLConnected_bool
            RTSP_Source_GetAudioDevices_str
            RTSP_Source_SetAudioDevice_str
            RTSP_Source_GetURL_str
            RTSP_Source_GetState_int

            RTSP_ONVIF_LastJPEGSnapshotBuffer_intptr
            RTSP_ONVIF_LastJPEGSnapshotSize_int

            RTSP_VideoStream_Enabled_bool
            RTSP_VideoStream_Synchronized_bool
            RTSP_VideoStream_Recorded_bool
            RTSP_VideoStream_Index_int
            RTSP_VideoStream_PinFormat_str
            RTSP_VideoStream_Width_int
            RTSP_VideoStream_Height_int
            RTSP_VideoStream_AspectRatio_double
            RTSP_VideoStream_TopDown_bool
            RTSP_VideoStream_FrameRateMax_double
            RTSP_VideoStream_Filter_str
            RTSP_VideoStream_HWAcceleration_int
            RTSP_VideoStream_Deinterlacing_int
            RTSP_VideoStream_ConfigureTextOverlay_str
            RTSP_VideoStream_ConfigureHueBrightSat_str

            RTSP_AudioStream_Enabled_bool
            RTSP_AudioStream_Recorded_bool
            RTSP_AudioStream_Index_int
            RTSP_AudioStream_Filter_str
            RTSP_AudioStream_Volume_int

            RTSP_FrameCapture_Width_int
            RTSP_FrameCapture_Height_int
            RTSP_FrameCapture_Time_int
            RTSP_FrameCapture_FileName_str

            RTSP_CurrentRecording_FileSizeKb_int
            RTSP_CurrentRecording_ClipDurationMs_int
            RTSP_CurrentRecording_VideoFrameCount_int
            RTSP_CurrentRecording_FileName_str

            RTSP_LastRecorded_FileSizeKb_int
            RTSP_LastRecorded_ClipDurationMs_int
            RTSP_LastRecorded_VideoFrameCount_int
            RTSP_LastRecorded_FileName_str

            RTSP_Dest_URL_str
            RTSP_Dest_Video_BitRate_int
            RTSP_Dest_Video_Quality_int
            RTSP_Dest_Video_KeyFrameInterval_int

            RTSP_Filter_Version_int
            RTSP_Filter_Build_int
            RTSP_Filter_LicenseKey_str
        End Enum

        Public Delegate Sub OpenURLAsyncCompletionCB(ByVal Sender As IntPtr, ByVal CustomParam As IntPtr, ByVal Result As UInt32)

        <ComImport(), Guid("58D66E77-B19C-4F9A-8B47-DFE1CA87B642"), System.Security.SuppressUnmanagedCodeSecurity(), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
        Public Interface IDatasteadRTSPSourceCallbackConfig
            <PreserveSig()> _
            Function SetAsyncOpenURLCallback(ByVal CallbackFunctionPtr As OpenURLAsyncCompletionCB, ByVal Sender As IntPtr, ByVal CustomParam As IntPtr) As Integer
        End Interface

        '* <summary> 
        '        <para>type of the sample callback function</para>
        '        

        <UnmanagedFunctionPointer(CallingConvention.StdCall)> _
        Public Delegate Sub DatasteadRTSP_RawSampleCallback(ByVal Sender As IntPtr, ByVal StreamNumber As Int32, ByVal CodecName As String, ByVal SampleTime_Absolute As Int64, ByVal SampleTime_Relative As Int64, ByVal Buffer As IntPtr, _
        ByVal BufferSize As Int32, ByVal InfoString As String)
        Public Delegate Sub DatasteadRTSP_VideoSampleCallback(ByVal Sender As IntPtr, ByVal SampleTime_Absolute As Int64, ByVal SampleTime_Relative As Int64, ByVal Buffer As IntPtr, ByVal BufferSize As Int32, ByVal VideoWidth As Int32, _
        ByVal VideoHeight As Int32, ByVal BitCount As Int32, ByVal Stride As Int32)
        Public Delegate Sub DatasteadRTSP_AudioSampleCallback(ByVal Sender As IntPtr, ByVal SampleTime_Absolute As Int64, ByVal SampleTime_Relative As Int64, ByVal Buffer As IntPtr, ByVal BufferSize As Int32, ByVal SamplesPerSec As Int32, _
        ByVal Channels As Int32, ByVal BitsPerSample As Int32)

        '* <summary> 
        '        <para>uncompressed audio and video sample callbacks</para>
        '        <para>Makes and return a copy of each sample uncompressed</para>
        '        

        <ComImport(), Guid("799485B3-0DA1-4F74-90E2-5684C9CD949B"), System.Security.SuppressUnmanagedCodeSecurity(), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
        Public Interface IDatasteadRTSPSampleCallback
            '* <summary> 
            '            <para>this function is used to configure the callback that occurs after invoking Action (RTSP_Action_OpenURLAsync, ...) when the URL opening completes. Parameters:</para>
            '            <para>DatasteadRTSPSampleCallback: address of your callback function</para>
            '            <para>Sender: pass a pointer to the current instance of your class, it will be returned as parameter when the callback occurs</para>
            '            

            <PreserveSig()> _
            Function SetRawSampleCallback(ByVal SampleCallback As IntPtr, ByVal Sender As IntPtr) As Integer
            Function SetVideoRGBSampleCallback(ByVal SampleCallback As IntPtr, ByVal Sender As IntPtr) As Integer
            Function SetAudioPCMSampleCallback(ByVal SampleCallback As IntPtr, ByVal Sender As IntPtr) As Integer
        End Interface

        '* <summary> 
        '        <para>direct access to the uncompressed RGB video sample before it is delivered to the output pin</para>
        '        <para>From this callback you can modify the video sample (e.g. apply an overlay) before it is delived</para>
        '        

        <ComImport(), Guid("845F8BAF-A774-4C46-A17A-F037613B9FC2"), System.Security.SuppressUnmanagedCodeSecurity(), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)> _
        Public Interface IDatasteadRTSPSampleCallback2
            '* <summary> 
            '            <para>this function is used to configure the callback that occurs after invoking Action (RTSP_Action_OpenURLAsync, ...) when the URL opening completes. Parameters:</para>
            '            <para>DatasteadRTSPSampleCallback: address of your callback function</para>
            '            <para>Sender: pass a pointer to the current instance of your class, it will be returned as parameter when the callback occurs</para>
            '            

            <PreserveSig()> _
            Function SetVideoRGBOverlayCallback(ByVal SampleCallback As IntPtr, ByVal Sender As IntPtr) As Integer
        End Interface

    End Class
End Namespace

