//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <activex.hpp>
#include <comobj.hpp>

#include "..\..\..\include\C++\DatasteadRTSPSourceFilter.h"

#include "DSPack_DatasteadRTSPSource_.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "DSPack"
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
	: TForm(Owner)
{
   DatasteadRTSPSourceConfig = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::RunGraph()
{
  POleStr OleStr;
  ICaptureGraphBuilder2 *CaptureGraphBuilder2;
  int BufferValue;
  HRESULT hr;
  HRESULT hrVid, hrAud;

   ClearGraph();
   FilterGraph->Mode = gmCapture;
   if (! FilterGraph->Active) FilterGraph->Active = true;

   if (FAILED(filterRTSP->QueryInterface (IID_IDatasteadRTSPSourceConfig,  reinterpret_cast<void**>(&DatasteadRTSPSourceConfig)))) return;

   BufferValue = StrToIntDef (edtBuffering->Text, 0);
   if (BufferValue > 0) {
      hr = DatasteadRTSPSourceConfig->SetInt (RTSP_Source_BufferDuration_int, BufferValue);
      LogErrorIfAny ("failed set buffering", hr);
   }

   if (chkRecord->Checked) {
      OleStr = StringToOleStr (edtRecordFileName->Text);
      hr = DatasteadRTSPSourceConfig->SetStr(RTSP_Source_RecordingFileName_str, OleStr);
      if (hr = S_OK) {
       btnGenerateNewFile->Enabled = true;
      }
      SysFreeString (OleStr);
      LogErrorIfAny ("failed set recording file name", hr);
   }

   if (edtUserName->Text > "") {
      OleStr = StringToOleStr (edtUserName->Text);
      hr = DatasteadRTSPSourceConfig->SetStr(RTSP_Source_AuthUser_str, OleStr);
      SysFreeString (OleStr);
      LogErrorIfAny ("failed set user name", hr);
   }

   if (edtPassword->Text > "") {
      OleStr = StringToOleStr (edtPassword->Text);
      hr = DatasteadRTSPSourceConfig->SetStr(RTSP_Source_AuthPassword_str, OleStr);
      SysFreeString (OleStr);
      LogErrorIfAny ("failed set password", hr);
   }

   OleStr = StringToOleStr (edtURL->Text);
   //MessageBoxW (0, OleStr, L"", 0);
   hr = DatasteadRTSPSourceConfig->Action (RTSP_Action_OpenURL, OleStr);
   SysFreeString (OleStr);
   LogErrorIfAny ("failed to open RTSP URL", hr);

   if (hr == S_OK) {
      if (SUCCEEDED (FilterGraph->QueryInterface(IID_ICaptureGraphBuilder2, reinterpret_cast<void**>(&CaptureGraphBuilder2)))) {
         hrVid = CaptureGraphBuilder2->RenderStream(NULL, &MEDIATYPE_Video, filterRTSP->operator _di_IFilter(), NULL, VideoWindow->operator _di_IFilter());
         hrAud = CaptureGraphBuilder2->RenderStream(NULL, &MEDIATYPE_Audio, filterRTSP->operator _di_IFilter(), NULL, NULL);
         if ((hrVid != S_OK) && (hrAud != S_OK)) {
            LogErrorIfAny ("failed to render URL streams", hrVid);
            hr = E_FAIL;
         }
      }
   }
   if (hr == S_OK) {
    FilterGraph->Play();
   }
   else {
    ClearGraph();
   }

}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnRUNClick(TObject *Sender)
{
	RunGraph();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::LogErrorIfAny(String LogInfo, HRESULT hr)
{
   if (hr != S_OK) {
      TVarRec LogInfoVars[] = {LogInfo, hr};
	   mmolog->Lines->Add (Format ("%s: error %08X", LogInfoVars, (ARRAYSIZE(LogInfoVars) -1)));
   }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::btnGenerateNewFileClick(TObject *Sender)
{
   POleStr OleStr;
   HRESULT hr;
   if (DatasteadRTSPSourceConfig) {
      OleStr = StringToOleStr (edtRecordFileName->Text);
      hr = DatasteadRTSPSourceConfig->Action (RTSP_Action_RecordToNewFileNow, OleStr);
      SysFreeString (OleStr);
      if (hr == S_OK) {
         mmolog->Lines->Add ("recording now to " + edtRecordFileName->Text);
      }
      else {
         LogErrorIfAny ("failed to generate new file", hr);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormCreate(TObject *Sender)
{
   char tempFolder[MAX_PATH];

   GetTempPath(MAX_PATH, tempFolder);
   String strtext = tempFolder;
   edtRecordFileName->Text = strtext + "record.mp4";

}
//---------------------------------------------------------------------------
void __fastcall TForm1::ClearGraph()
{
   if (DatasteadRTSPSourceConfig) DatasteadRTSPSourceConfig->Release();
   DatasteadRTSPSourceConfig = NULL;
   FilterGraph->ClearGraph();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   ClearGraph();
}
//---------------------------------------------------------------------------

