unit Main_Form;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, StdCtrls, ComCtrls, Buttons, ExtCtrls, ActiveX, DSPack, DSPack_DirectShow9;

{$include '..\..\..\include\Delphi\DatasteadRTSPSourceFilter.pas'}

type
  TMainForm = class(TForm)
    FilterGraph: TFilterGraph;
    VideoWindow: TVideoWindow;
    mmolog: TMemo;
    Memo1: TMemo;
    edtURL: TEdit;
    edtRecordFileName: TEdit;
    edtPassword: TEdit;
    edtUserName: TEdit;
    edtBuffering: TEdit;
    Label2: TLabel;
    chkRecord: TCheckBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    btnRUN: TButton;
    btnGenerateNewFile: TButton;
    Memo2: TMemo;
    mmoStreamInfo: TMemo;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnRUNClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnGenerateNewFileClick(Sender: TObject);

  private
     DatasteadRTSPSourceConfig: IDatasteadRTSPSourceConfig;
     function RunGraph(): Boolean;
     procedure ClearGraph();
     procedure LogErrorIfAny (LogInfo: string; hr: HRESULT);
  public

  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.btnRUNClick(Sender: TObject);
begin
   RunGraph();
end;

function TMainForm.RunGraph(): Boolean;
const
   CLSID_DatasteadRTSP : TGUID = '{55D1139D-5E0D-4123-9AED-575D7B039569}';
var
  OleStr: pOleStr;
  CaptureGraphBuilder2: ICaptureGraphBuilder2;
  BufferValue: LongInt;
  RTSPBaseFilter: IBaseFilter;
  hr: HRESULT;
  hrVid, hrAud: HRESULT;
begin
    Result := false;
    ClearGraph();
    try
       FilterGraph.Mode := gmCapture;
       if not FilterGraph.Active then FilterGraph.Active := true;

       if CoCreateInstance(CLSID_DatasteadRTSP, Nil, CLSCTX_INPROC_SERVER, IID_IBaseFilter, RTSPBaseFilter) <> S_OK then Exit;

       if (FilterGraph as IFilterGraph2).AddFilter (RTSPBaseFilter, 'DatasteadRTSPFilter') <> S_OK then Exit;

       DatasteadRTSPSourceConfig := RTSPBaseFilter as IDatasteadRTSPSourceConfig;

       BufferValue := StrToIntDef (edtBuffering.Text, 0);
       if BufferValue > 0 then begin
          hr := DatasteadRTSPSourceConfig.SetInt (RTSP_Source_BufferDuration_int, BufferValue);
          LogErrorIfAny ('failed set buffering', hr);
       end;

       if chkRecord.Checked then begin
          OleStr := StringToOleStr (edtRecordFileName.Text);
          hr := DatasteadRTSPSourceConfig.SetStr(RTSP_Source_RecordingFileName_str, OleStr);
          if hr = S_OK then begin
             btnGenerateNewFile.Enabled := true;
          end;
          SysFreeString (OleStr);
          LogErrorIfAny ('failed set recording file name', hr);
       end;

       if edtUserName.Text > '' then begin
         OleStr := StringToOleStr (edtUserName.Text);
          hr := DatasteadRTSPSourceConfig.SetStr(RTSP_Source_AuthUser_str, OleStr);
          SysFreeString (OleStr);
          LogErrorIfAny ('failed set user name', hr);
       end;

       if edtPassword.Text > '' then begin
          OleStr := StringToOleStr (edtPassword.Text);
          hr := DatasteadRTSPSourceConfig.SetStr(RTSP_Source_AuthPassword_str, OleStr);
          SysFreeString (OleStr);
          LogErrorIfAny ('failed set password', hr);
       end;

       OleStr := StringToOleStr (edtURL.Text);
       hr := DatasteadRTSPSourceConfig.Action (RTSP_Action_OpenURL, OleStr);
       SysFreeString (OleStr);
       LogErrorIfAny ('failed to open RTSP URL', hr);

       if hr = S_OK then begin
          if FilterGraph.QueryInterface(ICaptureGraphBuilder2, CaptureGraphBuilder2) = S_OK then begin
             hrVid := CaptureGraphBuilder2.RenderStream(nil, @MEDIATYPE_Video, RTSPBaseFilter, nil, VideoWindow as IBaseFilter);
             hrAud := CaptureGraphBuilder2.RenderStream(nil, @MEDIATYPE_Audio, RTSPBaseFilter, nil, nil);
             if (hrVid <> S_OK) and (hrAud <> S_OK) then begin
                LogErrorIfAny ('failed to render URL streams', hrVid);
                hr := E_FAIL;
             end;
          end;
       end;
       Result := hr = S_OK;
    finally
       if Result then begin
          FilterGraph.Play;
       end
       else begin
          ClearGraph();
       end;
    end;
end;

procedure TMainForm.LogErrorIfAny (LogInfo: string; hr: HRESULT);
begin
   if hr <> S_OK then begin
      mmoLog.Lines.Add (Format ('%s: error %08X', [LogInfo, hr]));
   end;
end;

procedure TMainForm.btnGenerateNewFileClick(Sender: TObject);
var
   OleStr: POleStr;
   hr: HRESULT;
begin
   if assigned (DatasteadRTSPSourceConfig) then begin
      OleStr := StringToOleStr (edtRecordFileName.Text);
      hr := DatasteadRTSPSourceConfig.Action (RTSP_Action_RecordToNewFileNow, OleStr);
      SysFreeString (OleStr);
      if hr = S_OK then begin
         mmoLog.Lines.Add ('recording now to ' + edtRecordFileName.Text);
      end
      else begin
         LogErrorIfAny ('failed to generate new file', hr);
      end;
   end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
   tempFolder: array[0..MAX_PATH] of Char;
begin
   GetTempPath(MAX_PATH, @tempFolder);
   EdtRecordFileName.Text := tempFolder + 'record.mp4';
end;

procedure TMainForm.ClearGraph();
begin
   if assigned (DatasteadRTSPSourceConfig) then DatasteadRTSPSourceConfig := nil;
   FilterGraph.ClearGraph;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  ClearGraph();
end;

end.
