﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.IO;

using DirectShowLib;
using Datastead;
using DatasteadDirectShow;

#if !USING_NET11
using System.Runtime.InteropServices.ComTypes;
#endif

// Note: static void Main() in Program.cs has the [STAThread] declaration, normally required for COM

namespace DatasteadRTSPSource_CSharp_Demo
{

    public partial class OpenURLForm : Form
    {
        public DatasteadDirectShowGraph_WithEventsMsg m_Graph = null;


        // It is possible to specify a number of seconds that must be included at the beginning of the recording, BEFORE RecordToNewFileNow has been invoked.
        // To use this feature the filter must be started with the recording "paused" (by passing "nul.mp4" as recording file name), then the recording is resumed 
        // when needed by invoking RecordToNewFileNow(newfilename.mp4).
        // This is designed to additionally include in the video clip the few seconds of video just before the user decided to start the recording.
        private int m_backtimed_recording_start_in_seconds = 0;


        public OpenURLForm()
        {
            InitializeComponent();

            m_Graph = new DatasteadDirectShowGraph_WithEventsMsg();
            m_Graph.m_AddToLog = AddToLog;
            m_Graph.m_PictureFromMemoryBitmap = imgCaptureToMemoryBitmap;
            m_Graph.m_DisplayWindow = panel1;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            tbRecordingFileName.Text = Path.GetTempPath() + "record.mp4";
            rdgDatasteadVideoRenderer.Checked = true;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_Graph.CloseInterfaces();
        }

        delegate void AddToLogCallback(string LogString);

        private void AddToLog(string LogString)
        {
            if (this.tbStreamInfo.InvokeRequired)
            {
                AddToLogCallback log = new AddToLogCallback(AddToLog);
                this.Invoke(log, new object[] { LogString });
            }
            else
            {
                this.tbStreamInfo.Text += Environment.NewLine + Environment.NewLine + LogString;
                this.tbStreamInfo.SelectionStart = tbStreamInfo.Text.Length;
                this.tbStreamInfo.ScrollToCaret();
            }
        }

        private void LogIfError(int hr, string LogString)
        {
            UInt32 HResult = (UInt32) hr;
            if (HResult > 0x80000000) // In DirectShow errors correspond to an HResult hex value >= 0x80000000
            {
                AddToLog(LogString + ": error 0x" + HResult.ToString("X8"));
            }
        }

        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {
                case DatasteadDirectShowGraph_WithEventsMsg.WM_GRAPHNOTIFY:
                    {
                        m_Graph.HandleGraphEvent();
                        break;
                    }
            }
            base.WndProc(ref m);
        }

        private void RTSP_VideoSampleCallback(IntPtr Sender, Int64 SampleTime_Absolute, Int64 SampleTime_Relative, IntPtr Buffer, Int32 BufferSize, Int32 VideoWidth, Int32 VideoHeight, Int32 BitCount, Int32 Stride)
        {
            // you can get from this event a sample callback for each RGB video frame
            // TO ACTIVATE THIS EVENT, UNCOMMENT THE 2 LINES BELOW RELATED TO "VideoSampleCallback"
        }

        private void RTSP_AudioSampleCallback(IntPtr Sender, Int64 SampleTime_Absolute, Int64 SampleTime_Relative, IntPtr Buffer, Int32 BufferSize, Int32 SamplesPerSec, Int32 Channels, Int32 BitsPerSample)
        {
            // you can get from this event a sample callback for each PCM audio sample.
            // TO ACTIVATE THIS EVENT, UNCOMMENT THE 2 LINES BELOW RELATED TO "AudioSampleCallback"
        }

        private void RTSP_RawSampleCallback(IntPtr Sender, Int32 StreamNumber, string CodecName, Int64 SampleTime_Absolute, Int64 SampleTime_Relative, IntPtr Buffer, Int32 BufferSize, string InfoString)
        {
            // you can get from this event a sample callback for each sample.
            // TO ACTIVATE THIS EVENT, UNCOMMENT THE 2 LINES BELOW RELATED TO "RawSampleCallback"
        }

        public bool OpenURL(bool OpenAsynchronously)
        {
            Application.CurrentCulture = System.Globalization.CultureInfo.InvariantCulture; // to get a dot as decimal separator when entering the frame rate

            //try
            {
                m_Graph.GetStandardDirectShowInterfaces(this.Handle);

                m_Graph.AddToROT(); // to be able to spy the graph from GraphEdit, remove this statement for production


                if (!m_Graph.AddDatasteadFilterAndQueryFilterInterfaces()) { 
                    AddToLog("error: failed to create Datastead RTSP filter instance");
                }
                else {
                    /* RGB video sample callback: uncomment the 2 lines below to activate it */
                    //RTSPFilter.DatasteadRTSP_VideoSampleCallback VideoSampleCallback = new RTSPFilter.DatasteadRTSP_VideoSampleCallback(RTSP_VideoSampleCallback);
                    //m_Graph.m_DatasteadRTSPSampleCallback.SetVideoRGBSampleCallback (Marshal.GetFunctionPointerForDelegate(VideoSampleCallback), IntPtr.Zero);

                    /* PCM audio sample callback: uncomment the 2 lines below to activate it */
                    //RTSPFilter.DatasteadRTSP_AudioSampleCallback AudioSampleCallback = new RTSPFilter.DatasteadRTSP_AudioSampleCallback(RTSP_AudioSampleCallback);
                    //m_Graph.m_DatasteadRTSPSampleCallback.SetAudioPCMSampleCallback (Marshal.GetFunctionPointerForDelegate(AudioSampleCallback), IntPtr.Zero);

                    /* raw sample callback: uncomment the 2 lines below to activate it */
                    //RTSPFilter.DatasteadRTSP_RawSampleCallback RawSampleCallback = new RTSPFilter.DatasteadRTSP_RawSampleCallback(RTSP_RawSampleCallback);
                    //m_Graph.m_DatasteadRTSPSampleCallback.SetRawSampleCallback(Marshal.GetFunctionPointerForDelegate(RawSampleCallback), IntPtr.Zero);

                    m_Graph.m_DatasteadRTSPConfigHelper.SetBool(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_Enabled_bool, chkVideoStreamEnabled.Checked);

                    m_Graph.m_DatasteadRTSPConfigHelper.SetBool(RTSPFilter.RTSPConfigParam.RTSP_AudioStream_Enabled_bool, chkAudioStreamEnabled.Checked);

                    m_Graph.m_DatasteadRTSPConfigHelper.SetBool(RTSPFilter.RTSPConfigParam.RTSP_AudioStream_Recorded_bool, chkAudioStreamRecorded.Checked);

                    m_Graph.m_DatasteadRTSPConfigHelper.SetBool(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_Recorded_bool, chkVideoStreamRecorded.Checked);

                    m_Graph.m_DatasteadRTSPConfigHelper.SetBool(RTSPFilter.RTSPConfigParam.RTSP_Source_AutoReconnect_bool, chkAutoReconnect.Checked);

                    if (chkEnableTextOverlay.Checked)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureTextOverlay_str, txtTextOverlay.Text);
                        m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureTextOverlay_str, txtTextOverlay2.Text);
                        // it is possible to add more overlays
                    }

                    if (chkBrightHueSat.Checked)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureHueBrightSat_str, txtBrightHueSat.Text);
                    }

                    if (rdbTransportDefault.Checked)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, RTSPFilter.RTSPTransportMode_Auto);
                    }
                    else if (rdbTransportUDP.Checked)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, RTSPFilter.RTSPTransportMode_UDP);
                    }
                    else if (rdbTransportTCP.Checked)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, RTSPFilter.RTSPTransportMode_TCP);
                    }
                    else if (rdbTransportHTTP.Checked)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, RTSPFilter.RTSPTransportMode_HTTP);
                    }
                    else if (rdbTransportMulticast.Checked)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, RTSPFilter.RTSPTransportMode_Udp_Multicast);
                    }

                    if (chkVidSync.CheckState != CheckState.Indeterminate)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetBool(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_Synchronized_bool, chkVidSync.Checked);
                    }

                    if (chkLowDelay.CheckState != CheckState.Indeterminate)
                    {
                        m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_LowDelay_int, Convert.ToInt32(chkLowDelay.Checked)); // 0 = disabled, 1 = enabled
                    }

                    if (chkUseSelectedAudioDeviceAsAudioSource.Checked)
                    {
                        if (lstAudioDevices.Items.Count > 0)
                        {
                            if (lstAudioDevices.SelectedIndex > -1)
                            {
                                string SelectedAudioDevice = lstAudioDevices.Items[lstAudioDevices.SelectedIndex].ToString();
                                m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_Source_SetAudioDevice_str, SelectedAudioDevice);
                            }
                        }
                    }

                    int BufferValue;
                    if (Int32.TryParse(tbBuffering.Text, out BufferValue))
                    {
                        int hr =m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_BufferDuration_int, BufferValue);
                        LogIfError (hr, "set buffer duration");
                    }

                    int TimePositionValue;
                    if (Int32.TryParse(tbTimePosition.Text, out TimePositionValue))
                    {
                        if (TimePositionValue > 0)
                        {
                            int hr =m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_StartTime_int, TimePositionValue);
                            LogIfError (hr, "set start time");
                        }
                    }

                    double FrameRate;
                    if (double.TryParse(tbFrameRate.Text, out FrameRate))
                    {
                        int hr =m_Graph.m_DatasteadRTSPConfigHelper.SetDouble(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_FrameRateMax_double, FrameRate);
                        LogIfError (hr, "set frame rate");
                    }

                    if (tbUserName.Text != string.Empty)
                    {
                        int hr =m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_Source_AuthUser_str, tbUserName.Text);
                        LogIfError (hr, "set user authentication");
                    }

                    if (tbPassword.Text != string.Empty)
                    {
                        int hr =m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_Source_AuthPassword_str, tbPassword.Text);
                        LogIfError (hr, "set password authentication");
                    }

                    if (chkRestreamURL.Checked)
                    {
                        int hr =m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_Dest_URL_str, edtRestreamURL.Text);
                        LogIfError (hr, "set restream URL");
                    }


                    if (cbRecord.Checked) {
                        int hr = m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_Source_RecordingFileName_str, tbRecordingFileName.Text);
                        LogIfError(hr, "set recording file name");

                        if (m_backtimed_recording_start_in_seconds > 0)
                        {
                            hr = m_Graph.m_DatasteadRTSPConfigHelper.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_RecordingBacktimedStartSeconds_int, m_backtimed_recording_start_in_seconds);
                            LogIfError(hr, "set backtimed recording start in seconds");
                            hr = m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_PauseRecording, "");
                            LogIfError(hr, "put the recording in paused mode"); // will be resumed when needed while the graph is running by invoking Action(RTSP_Action_ResumeRecording, "")
                            MessageBox.Show(hr.ToString());
                        }
                    }

                    string URL = tbURL.Text;

                    m_Graph.GraphRunResult = GraphRunResultCallback;

                    if (OpenAsynchronously)
                    {
                        // Action (RTSP_Action_OpenURLAsync) opens the URL asynchrnously
                        int hr =m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_OpenURLAsync, URL);
                        LogIfError(hr, "open URL asynchronously");
                        /// starting from here the filter connects asynchrnously to the URL and notify the graph through IMediaEventEx by sending EC_RTSPNOTIFY (EC_RTSP_PARAM1_OPENURLASYNC_SUCCEEDED, 0), see the HandleGraphEvent function below
                        if (hr == 0)
                        {
                            AddToLog("Connecting asynchronously...");
                        }
                    }
                    else
                    {
                        // Action (RTSP_Action_OpenURL) opens the URL synchrnously
                        AddToLog("Connecting synchronously...");
                        Application.DoEvents(); /// to update the log window
                        int hr =m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_OpenURL, URL);
                        LogIfError(hr, "open URL synchronously");
                        if (hr == 0)
                        {
                            m_Graph.RunGraph();
                        }
                    }
                }
            }
            return true;
        }

        public void GraphRunResultCallback(bool Success)
        {
            AddToLog("running");
            btnPauseRecording.Enabled = cbRecord.Checked;
            btnResumeRecording.Enabled = cbRecord.Checked;
            btnGenerateNewFile.Enabled = cbRecord.Checked;
        }

        private void Form1_Resize(object sender, System.EventArgs e)
        {
            m_Graph.ResizeVideoWindow();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            m_Graph.CloseInterfaces();
            AddToLog("closed.");
        }

        private void btnInfoLimitations_Click(object sender, EventArgs e)
        {
            const string info = "- LIMITATIONS OF THE EVALUATION VERSION:\r\n- several filter instances can be run concurrently in the same application, each instance will stop randomly and independently after a few minutes to a few hours.\r\n- Once one of the instances has stopped, no other instance can be (re)started until the application is restarted.";
            MessageBox.Show(info);

        }

        private void btnGenerateNewFile_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                // if you pass string.Empty as 2nd parameter the current file being recorded will be overwritten
                int hr = m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_RecordToNewFileNow, tbRecordingFileName.Text);
                LogIfError(hr, "record to new file failed");
            }
        }

        private void btnOpenSync_Click(object sender, EventArgs e)
        {
            m_Graph.CloseInterfaces();
            OpenURL(false);
            // starting from here the filter connects asynchrously, 
            // then when the connection completes the filter sends
            // an EC_RTSP_NOTIFY event to the graph event handler (HandleGraphEvent) 
            // that invokes CompleteGraph()
            // >>>>> SEE HandleGraphEvent() and CompleteGraph() <<<<<
        }

        private void btnOpenAsync_Click(object sender, EventArgs e)
        {
            m_Graph.CloseInterfaces();
            OpenURL(true);
            // starting from here the filter is connected and the graph running
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            m_Graph.CloseInterfaces();
        }

        private void btnPause_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_PauseURL, "");
            }
        }

        private void btnResume_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_ResumeURL, "");
            }
        }

        private void btnBufferInfo_Click(object sender, EventArgs e)
        {
            MessageBox.Show ("Specify explicitely 0 to disable the default buffering. If the video freezes periodically, increase the value, e.g. between 200 and 1000 ms, or disable the video sync if not required");
        }

        private void btnRestreamInfo_Click(object sender, EventArgs e)
        {
            MessageBox.Show("destination IP:port URL to which the video will be re-streamed:\n\n" +
            "- UDP unicast, e.g.:\n" + 
            "udp://192.168.2:45:10000\n\n" +
            "- UDP multicast, e.g.:\n" +
            "udp://239.255.0.10:11000\n\n" +
            "- RTSP server, e.g.:\n" +
            "rtspsrv://192.168.0.22:12000");
        }

        private void btnCaptureToJPG_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "snapshot.jpg");
            }
        }

        private void btnCaptureToPNG_Click(object sender, EventArgs e)
        {
            if (IntPtr.Size > 4)
            {
                MessageBox.Show(".png format not yet supported in x64 build, this will be fixed in future versions, for the moment use the x86 build or choose another format");
            }
            else
            {
                if (m_Graph.m_DatasteadRTSPSourceConfig != null)
                {
                    m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "snapshot.png");
                }
            }
        }

        private void btnCaptureToTIFF_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "snapshot.tiff");
           }

        }

        private void btnCaptureToBMP_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "snapshot.bmp");
            }
        }

        private void btnCaptureToMemoryBitmap_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "HBITMAP"); // "HBITMAP" is a reserved keyword. For the corresponding sample code search for "m_PictureFromMemoryBitmap" in this project
            }
        }

        private void btnGetAudioDevicesList_Click(object sender, EventArgs e)
        {
            lstAudioDevices.Items.AddRange (m_Graph.GetAudioDevicesAsItems());
        }

        private void btnGenerateNewFileInfo_Click(object sender, EventArgs e)
        {
            MessageBox.Show ("To pause the recording, enter a file name with the same extension, but \"nul\" as name and no file path, e.g. \"nul.mp4\".  If the new file name is identical the previous recording is overwritten");
        }

        private void txtTextOverlay_TextChanged(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null) // we can update the overlay in real time
            {
                m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureTextOverlay_str, ((TextBox) sender).Text);
            }
            Warn_TextOverlay(sender, e);
        }

        private void txtBrightHueSat_TextChanged(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null) // we can update the hue, brightness and saturation in real time
            {
                m_Graph.m_DatasteadRTSPConfigHelper.SetStr(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureHueBrightSat_str, txtBrightHueSat.Text);
            }
            Warn_BrightHueSat(sender, e);
        }

        private bool m_TextOverlayWarningNotified = false;
        private void Warn_TextOverlay(object sender, EventArgs e)
        {
            if (!m_TextOverlayWarningNotified)
            {
                m_TextOverlayWarningNotified = true;
                AddToLog("Warning: each overlay must be set BEFORE LOADING the URL. To prevent displaying an overlay immediately, set an empty text string, then update the text string when needed, while the filter is running.");
                AddToLog("Warning: this edit field is for quick and easier demonstration purpose only. An incorrect syntax (e.g. wrong color name) may crash the filter, especially when updating it in real time.");
            }
        }
        private bool m_BrightHueSatWarningNotified = false;
        private void Warn_BrightHueSat(object sender, EventArgs e)
        {
            if (!m_BrightHueSatWarningNotified)
            {
                m_BrightHueSatWarningNotified = true;
                AddToLog("brightness: in the -10..10 range (default 0)");
                AddToLog("hue: in degrees (default 0)");
                AddToLog("saturation: in the -10..10 range (default 1)");
                AddToLog("Warning: the brightness/hue/saturation setting must be set BEFORE LOADING the URL. To prevent it to be applied immediately, set the default value(s) (b=0,h=0,s=1), then update them when needed while the filter is running.");
            }
        }

        private void chkVideoStreamEnabled_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_VideoStreamEnabled = chkVideoStreamEnabled.Checked;
        }

        private void chkAudioStreamEnabled_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_AudioStreamEnabled = chkAudioStreamEnabled.Checked;
        }

        private void chkVideoStreamVisible_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_VideoStreamVisible = chkVideoStreamVisible.Checked;
        }

        private void chkAudioStreamRendered_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_AudioStreamRendered = chkAudioStreamRendered.Checked;
        }

        private void rdgVideoRendererDefault_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_VideoRendererToUse = DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_Default;
        }

        private void rdgDatasteadVideoRenderer_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_VideoRendererToUse = DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_DatasteadRenderer;
        }

        private void rdgStandardVideoRenderer_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_VideoRendererToUse = DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_StandardRenderer;
        }

        private void rdgVideoRendererVMR7_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_VideoRendererToUse = DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_VMR7;
        }

        private void rdgVideoRendererVMR9_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_VideoRendererToUse = DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_VMR9;
        }

        private void rdgNullRenderer_CheckedChanged(object sender, EventArgs e)
        {
            m_Graph.m_VideoRendererToUse = DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_NullRenderer;
        }

        private void btnPauseRecording_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                // if you pass string.Empty as 2nd parameter the current file being recorded will be overwritten
                int hr = m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_PauseRecording, "");
                LogIfError(hr, "pause recording failed");
            }

        }
        private void btnResumeRecording_Click(object sender, EventArgs e)
        {
            if (m_Graph.m_DatasteadRTSPSourceConfig != null)
            {
                // if you pass string.Empty as 2nd parameter the current file being recorded will be overwritten
                int hr = m_Graph.m_DatasteadRTSPConfigHelper.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_ResumeRecording, "");
                LogIfError(hr, "pause recording failed");
            }
        }

        private void btnBacktimedStartInfo_Click(object sender, EventArgs e)
        {
            MessageBox.Show("It is possible to specify a number of seconds that must be included at the beginning of the recording, BEFORE RecordToNewFileNow or ResumeRecording has been invoked.\n\nTo use this feature the recording must be started in \"paused\" mode by invoking Action(RTSP_Action_PauseRecording) before starting the graph. \n\nThen to begin writing to the file invoke Action(RTSP_Action_ResumeRecording) or Action(RTSP_Action_RecordToNewFileNow, newfilename.mp4).\n\nThis will include in the video clip the specified number of seconds of video just before the user decided to start the recording.");

        }
        private void cbRecord_CheckedChanged(object sender, EventArgs e)
        {
            labelBacktimedSeconds.Enabled = cbRecord.Checked;
            txtBacktimedStartSeconds.Enabled = cbRecord.Checked;
            btnBacktimedRecSecondsOK.Enabled = cbRecord.Checked;

        }

        private void btnBacktimedRecSecondsOK_Click(object sender, EventArgs e)
        {
            m_backtimed_recording_start_in_seconds = 0;
            if (int.TryParse(txtBacktimedStartSeconds.Text, out m_backtimed_recording_start_in_seconds))
                if (m_backtimed_recording_start_in_seconds > 0)
                {
                    MessageBox.Show("the graph will start without writing to the file immediately, when you want to begin writing to the file, press \"Resume Recording\"");
                }
        }
    }
    
}
