﻿#define ASYNCOPEN
// comment the ASYNCOPEN define above to open the URL synchronously (by blocking the main thread)
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using DirectShowLib;
using Datastead;
using DatasteadDirectShow;

// Note: static void Main() in Program.cs has the [STAThread] declaration, normally required for COM

namespace VideoFramesToPictureBox
{
    public partial class Form1 : Form
    {
        DatasteadDirectShowGraph_WithEventsThread m_Graph = new DatasteadDirectShowGraph_WithEventsThread();

        private RTSPFilter.DatasteadRTSP_VideoSampleCallback m_VideoSampleCallback = null;

        public Form1()
        {
            InitializeComponent();
        }


        private void OpenURL_Click(object sender, EventArgs e)
        {
            m_Graph = new DatasteadDirectShowGraph_WithEventsThread();
            pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;

            m_Graph.CloseInterfaces();
            m_Graph.GetStandardDirectShowInterfaces();
            m_Graph.AddToROT(); // to be able to spy the graph from GraphEdit or GraphStudioNext, remove this statement for production

            m_Graph.m_VideoRendererToUse = DatasteadDirectShowGraph_WithEventsThread.VideoRendererToUse.vr_NullRenderer;

            if (m_Graph.AddDatasteadFilterAndQueryFilterInterfaces())
            {
                m_VideoSampleCallback = new RTSPFilter.DatasteadRTSP_VideoSampleCallback(RTSP_VideoSampleCallback);
                m_Graph.m_DatasteadRTSPSampleCallback.SetVideoRGBSampleCallback(Marshal.GetFunctionPointerForDelegate(m_VideoSampleCallback), IntPtr.Zero);

                m_Graph.m_DatasteadRTSPSourceConfig.SetBool(RTSPFilter.RTSPConfigParam.RTSP_VideoStream_TopDown_bool.ToString(), true);
                m_Graph.m_DatasteadRTSPSourceConfig.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_BufferDuration_int.ToString(), 500); // set a buffering of 500ms, can be reduced for a lower latency
                m_Graph.m_DatasteadRTSPSourceConfig.SetInt(RTSPFilter.RTSPConfigParam.RTSP_Source_LowDelay_int.ToString(), 1); // enables the low delay mode, remove this line if the video is not smooth
#if ASYNCOPEN
                m_Graph.m_DatasteadRTSPSourceConfig.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_OpenURLAsync.ToString(), URL.Text);
#else
                m_Graph.m_DatasteadRTSPSourceConfig.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_OpenURL.ToString(), URL.Text);
                m_Graph.RunGraph();
#endif
            }
        }

        private delegate void UpdatePictureBoxDelegate(Bitmap FrameReceived, int pictureboxheight);

        private void UpdatePictureBox(Bitmap FrameReceived, int pictureboxheight)
        {
            if (pictureboxheight != pictureBox1.Height)
            {
                pictureBox1.Height = pictureboxheight;
            }
            pictureBox1.Image = FrameReceived;
        }


        private void RTSP_VideoSampleCallback(IntPtr Sender, Int64 SampleTime_Absolute, Int64 SampleTime_Relative, IntPtr Buffer, Int32 BufferSize, Int32 VideoWidth, Int32 VideoHeight, Int32 BitCount, Int32 Stride)
        {
            Bitmap FrameReceived = null;
            if (BitCount == 32)
            {
                FrameReceived = new Bitmap(VideoWidth, VideoHeight, Stride, System.Drawing.Imaging.PixelFormat.Format32bppRgb, Buffer);
            }
            else if (BitCount == 24)
            {
                FrameReceived = new Bitmap(VideoWidth, VideoHeight, Stride, System.Drawing.Imaging.PixelFormat.Format24bppRgb, Buffer);
            }
            if (FrameReceived != null)
            {
                float ratio = (float)VideoHeight / (float)VideoWidth;
                float width = (float)pictureBox1.Width;
                float height = width * ratio;
                int pictureboxheight = (int)height;

                object[] myArray = new object[2];
                myArray[0] = FrameReceived;
                myArray[1] = pictureboxheight;

                pictureBox1.BeginInvoke(new UpdatePictureBoxDelegate(UpdatePictureBox), myArray);
            }
        }

        private void CloseURL_Click(object sender, EventArgs e)
        {
            m_Graph.CloseInterfaces();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_Graph.CloseInterfaces();
        }


    }
}
