﻿//#define CONNECT_ASYNCHRONOUSLY
// comment the CONNECT_ASYNCHRONOUSLY define above to open the URL synchronously (by blocking the main thread)
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.IO;

using DirectShowLib;
using Datastead;
using DatasteadDirectShow;

// Note: static void Main() in Program.cs has the [STAThread] declaration, normally required for COM

namespace ONVIFSnapshot
{
    public partial class Form1 : Form
    {
        DatasteadDirectShowGraph_WithEventsThread m_Graph;
        public Form1()
        {
            InitializeComponent();
            m_Graph = new DatasteadDirectShowGraph_WithEventsThread();
            m_Graph.m_AddToLog = AddToLog;
        }

        private void AddToLog(string LogString)
        {
            if (this.txtInfo.InvokeRequired)
            {
                DatasteadDirectShowGraph_WithEventsThread.AddToLog log = new DatasteadDirectShowGraph_WithEventsThread.AddToLog(AddToLog);
                this.Invoke(log, new object[] { LogString });
            }
            else
            {
                this.txtInfo.Text += Environment.NewLine + Environment.NewLine + LogString;
                this.txtInfo.SelectionStart = txtInfo.Text.Length;
                this.txtInfo.ScrollToCaret();
                this.txtInfo.Update();
            }
        }

        #if CONNECT_ASYNCHRONOUSLY

        private bool m_ConnectingAsynchronously = false;

        private void GraphEvent(int EventCode, int Param1, int Param2)
        {
            if (this.InvokeRequired)
            {
                DatasteadDirectShowGraph.GraphEvent ev = new DatasteadDirectShowGraph.GraphEvent(GraphEvent);
                this.Invoke(ev, new object[] { EventCode, Param1, Param2 });
            }
            else
            {
                if (EventCode == RTSPFilter.EC_RTSPNOTIFY)
                {
                    if (Param1 == RTSPFilter.EC_RTSP_PARAM1_ONVIF_SNAPSHOT_SUCCEEDED)
                    {
                        IntPtr pJPEGBuffer;
                        int JPEGBufferSize;

                        RTSPFilter.IDatasteadRTSPSourceConfig2 DatasteadRTSPSourceConfig2 = (RTSPFilter.IDatasteadRTSPSourceConfig2)m_Graph.m_DatasteadRTSPSourceConfig; // DatasteadRTSPSourceConfig2 exposes GetIntPtr, additionally to the IDatasteadRTSPSourceConfig methods 

                        DatasteadRTSPSourceConfig2.GetInt(RTSPFilter.RTSPConfigParam.RTSP_ONVIF_LastJPEGSnapshotSize_int.ToString(), out JPEGBufferSize);
                        DatasteadRTSPSourceConfig2.GetIntPtr(RTSPFilter.RTSPConfigParam.RTSP_ONVIF_LastJPEGSnapshotBuffer_intptr.ToString(), out pJPEGBuffer);

                        AddToLog("snapshot available in the pJPEGBuffer BYTE array, size=" + JPEGBufferSize.ToString());
                        m_ConnectingAsynchronously = false;
                        // snapshot captured
                    }
                    else if (Param1 == RTSPFilter.EC_RTSP_PARAM1_ONVIF_SNAPSHOT_FAILED)
                    {
                        AddToLog("snapshot failed");
                        m_ConnectingAsynchronously = false;
                    }
                }

            }
        }
        #endif

        private void btnGetONVIFSnapshot_Click(object sender, EventArgs e)
        {

            #if CONNECT_ASYNCHRONOUSLY
            if (m_ConnectingAsynchronously) return;
            m_ConnectingAsynchronously = true;
            m_Graph.m_GraphEventCallback = GraphEvent;
            #endif

            AddToLog("connecting...");

            m_Graph.CloseInterfaces();
            m_Graph.GetStandardDirectShowInterfaces();
            m_Graph.GraphCanRun = false; // NO NEED TO RUN THE GRAPH

            m_Graph.m_VideoRendererToUse = DatasteadDirectShowGraph_WithEventsThread.VideoRendererToUse.vr_None;

            if (m_Graph.AddDatasteadFilterAndQueryFilterInterfaces())
            {
                if (chkSaveSnapshotToFile.Checked)
                {
                    m_Graph.m_DatasteadRTSPSourceConfig.SetStr(RTSPFilter.RTSPConfigParam.RTSP_Source_RecordingFileName_str.ToString(), txtSnapshotFilename.Text);
                }

                if (txtUserName.Text != "")
                {
                    m_Graph.m_DatasteadRTSPSourceConfig.SetStr(RTSPFilter.RTSPConfigParam.RTSP_Source_AuthUser_str.ToString(), txtUserName.Text);
                }

                if (txtPassword.Text != "")
                {
                    m_Graph.m_DatasteadRTSPSourceConfig.SetStr(RTSPFilter.RTSPConfigParam.RTSP_Source_AuthPassword_str.ToString(), txtPassword.Text);
                }

                #if CONNECT_ASYNCHRONOUSLY
                m_Graph.m_DatasteadRTSPSourceConfig.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_GetONVIFSnapshotAsync.ToString(), URL.Text);
                #else
                if (m_Graph.m_DatasteadRTSPSourceConfig.Action(RTSPFilter.RTSPConfigParam.RTSP_Action_GetONVIFSnapshot.ToString(), URL.Text) == 0)
                {
                    IntPtr pJPEGBuffer;
                    int JPEGBufferSize = 0;
                    
                    RTSPFilter.IDatasteadRTSPSourceConfig2 DatasteadRTSPSourceConfig2 = (RTSPFilter.IDatasteadRTSPSourceConfig2) m_Graph.m_DatasteadRTSPSourceConfig; // DatasteadRTSPSourceConfig2 exposes GetIntPtr, additionally to the IDatasteadRTSPSourceConfig methods 
                    DatasteadRTSPSourceConfig2.GetInt (RTSPFilter.RTSPConfigParam.RTSP_ONVIF_LastJPEGSnapshotSize_int.ToString(), out JPEGBufferSize);
                    DatasteadRTSPSourceConfig2.GetIntPtr (RTSPFilter.RTSPConfigParam.RTSP_ONVIF_LastJPEGSnapshotBuffer_intptr.ToString(), out pJPEGBuffer);
                    AddToLog ("snapshot available in the pJPEGBuffer BYTE array, size=" + JPEGBufferSize.ToString());
                    // snapshot captured
                }
                else
                {
                    AddToLog ("snapshot failed");
                }
                #endif
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            txtSnapshotFilename.Text = Path.GetTempPath() + "snapshot.jpg";
        }

        private void Form1_Deactivate(object sender, EventArgs e)
        {
            m_Graph.CloseInterfaces();
        }



    }
}
