﻿
Imports System.Diagnostics
Imports System.Drawing
Imports System.Runtime.InteropServices
Imports System.Windows.Forms
Imports System.IO

Imports DirectShowLib
Imports Datastead
Imports DatasteadDirectShow


#If Not USING_NET11 Then
Imports System.Runtime.InteropServices.ComTypes
#End If

Public Class OpenURLForm
    Inherits Form

    Public m_Graph As DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg = New DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg()


    ' It is possible to specify a number of seconds that must be included at the beginning of the recording, BEFORE RecordToNewFileNow has been invoked.
    ' To use this feature the filter must be started with the recording "paused" (by passing "nul.mp4" as recording file name), then the recording is resumed 
    ' when needed by invoking RecordToNewFileNow(newfilename.mp4).
    ' This is designed to additionally include in the video clip the few seconds of video just before the user decided to start the recording.
    Private m_backtimed_recording_start_in_seconds As Integer = 0

    Public Sub New()
        InitializeComponent()

        m_Graph.m_AddToLog = AddressOf AddToLog
        m_Graph.m_PictureFromMemoryBitmap = imgCaptureToMemoryBitmap
        m_Graph.m_DisplayWindow = panel1
    End Sub

    ' normally STAThread is required for COM
    <STAThread()> _
    Public Shared Sub Main()
        Dim aform As New OpenURLForm()
        Application.Run(aform)
    End Sub

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs) Handles MyBase.Load
        tbRecordingFileName.Text = Path.GetTempPath() + "record.mp4"
        rdgDatasteadVideoRenderer.Checked = True
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As FormClosingEventArgs) Handles MyBase.Closing
        m_Graph.CloseInterfaces()
    End Sub

    Private Delegate Sub AddToLogCallback(ByVal LogString As String)

    Private Sub AddToLog(ByVal LogString As String)
        If IsHandleCreated Then
            If Me.tbStreamInfo.InvokeRequired Then
                Dim log As New AddToLogCallback(AddressOf AddToLog)
                Me.Invoke(log, New Object() {LogString})
            Else
                Me.tbStreamInfo.Text += Environment.NewLine + Environment.NewLine + LogString
                Me.tbStreamInfo.SelectionStart = tbStreamInfo.Text.Length
                Me.tbStreamInfo.ScrollToCaret()
            End If
        End If
    End Sub

    Private Sub LogIfError(ByVal hr As Integer, ByVal LogString As String)
        Dim HResult As UInt32 = Convert.ToUInt32(hr)
        If HResult > &H80000000UI Then
            ' In DirectShow errors correspond to an HResult hex value >= 0x80000000
            AddToLog((LogString & Convert.ToString(": error 0x")) + HResult.ToString("X8"))
        End If
    End Sub

    Protected Overrides Sub WndProc(ByRef m As Message)
        Select Case m.Msg
            Case DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg.WM_GRAPHNOTIFY
                If True Then
                    m_Graph.HandleGraphEvent()
                    Exit Select
                End If
        End Select
        MyBase.WndProc(m)
    End Sub

    Private Sub RTSP_VideoSampleCallback(ByVal Sender As IntPtr, ByVal SampleTime_Absolute As Int64, ByVal SampleTime_Relative As Int64, ByVal Buffer As IntPtr, ByVal BufferSize As Int32, ByVal VideoWidth As Int32, _
     ByVal VideoHeight As Int32, ByVal BitCount As Int32, ByVal Stride As Int32)
        ' you can get from this event a sample callback for each RGB video frame
        ' TO ACTIVATE THIS EVENT, UNCOMMENT THE 2 LINES BELOW RELATED TO "VideoSampleCallback"
    End Sub

    Private Sub RTSP_AudioSampleCallback(ByVal Sender As IntPtr, ByVal SampleTime_Absolute As Int64, ByVal SampleTime_Relative As Int64, ByVal Buffer As IntPtr, ByVal BufferSize As Int32, ByVal SamplesPerSec As Int32, _
     ByVal Channels As Int32, ByVal BitsPerSample As Int32)
        ' you can get from this event a sample callback for each PCM audio sample.
        ' TO ACTIVATE THIS EVENT, UNCOMMENT THE 2 LINES BELOW RELATED TO "AudioSampleCallback"
    End Sub

    Private Sub RTSP_RawSampleCallback(ByVal Sender As IntPtr, ByVal StreamNumber As Int32, ByVal CodecName As String, ByVal SampleTime_Absolute As Int64, ByVal SampleTime_Relative As Int64, ByVal Buffer As IntPtr, _
     ByVal BufferSize As Int32, ByVal InfoString As String)
        ' you can get from this event a sample callback for each sample.
        ' TO ACTIVATE THIS EVENT, UNCOMMENT THE 2 LINES BELOW RELATED TO "RawSampleCallback"
    End Sub

    Public Function OpenURL(ByVal OpenAsynchronously As Boolean) As Boolean
        Application.CurrentCulture = System.Globalization.CultureInfo.InvariantCulture
        ' to get a dot as decimal separator when entering the frame rate
        'try
        If True Then
            m_Graph.GetStandardDirectShowInterfaces(Me.Handle)

            m_Graph.AddToROT()
            ' to be able to spy the graph from GraphEdit, remove this statement for production

            If Not m_Graph.AddDatasteadFilterAndQueryFilterInterfaces() Then
                AddToLog("error: failed to create Datastead RTSP filter instance")
            Else
                ' RGB video sample callback: uncomment the 2 lines below to activate it 

                'RTSPFilter.DatasteadRTSP_VideoSampleCallback VideoSampleCallback = new RTSPFilter.DatasteadRTSP_VideoSampleCallback(RTSP_VideoSampleCallback);
                'm_Graph.m_DatasteadRTSPSampleCallback.SetVideoRGBSampleCallback (Marshal.GetFunctionPointerForDelegate(VideoSampleCallback), IntPtr.Zero);

                ' PCM audio sample callback: uncomment the 2 lines below to activate it 

                'RTSPFilter.DatasteadRTSP_AudioSampleCallback AudioSampleCallback = new RTSPFilter.DatasteadRTSP_AudioSampleCallback(RTSP_AudioSampleCallback);
                'm_Graph.m_DatasteadRTSPSampleCallback.SetAudioPCMSampleCallback (Marshal.GetFunctionPointerForDelegate(AudioSampleCallback), IntPtr.Zero);

                ' raw sample callback: uncomment the 2 lines below to activate it 

                'RTSPFilter.DatasteadRTSP_RawSampleCallback RawSampleCallback = new RTSPFilter.DatasteadRTSP_RawSampleCallback(RTSP_RawSampleCallback);
                'm_Graph.m_DatasteadRTSPSampleCallback.SetRawSampleCallback(Marshal.GetFunctionPointerForDelegate(RawSampleCallback), IntPtr.Zero);

                m_Graph.m_DatasteadRTSPConfigHelper.SetBool(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_Enabled_bool, chkVideoStreamEnabled.Checked)

                m_Graph.m_DatasteadRTSPConfigHelper.SetBool(Datastead.RTSPFilter.RTSPConfigParam.RTSP_AudioStream_Enabled_bool, chkAudioStreamEnabled.Checked)

                m_Graph.m_DatasteadRTSPConfigHelper.SetBool(Datastead.RTSPFilter.RTSPConfigParam.RTSP_AudioStream_Recorded_bool, chkAudioStreamRecorded.Checked)

                m_Graph.m_DatasteadRTSPConfigHelper.SetBool(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_Recorded_bool, chkVideoStreamRecorded.Checked)

                m_Graph.m_DatasteadRTSPConfigHelper.SetBool(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_AutoReconnect_bool, chkAutoReconnect.Checked)

                If chkEnableTextOverlay.Checked Then
                    m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureTextOverlay_str, txtTextOverlay.Text)
                    ' it is possible to add more overlays
                    m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureTextOverlay_str, txtTextOverlay2.Text)
                End If

                If chkBrightHueSat.Checked Then
                    m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureHueBrightSat_str, txtBrightHueSat.Text)
                End If

                If rdbTransportDefault.Checked Then
                    m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, Datastead.RTSPFilter.RTSPTransportMode_Auto)
                ElseIf rdbTransportUDP.Checked Then
                    m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, Datastead.RTSPFilter.RTSPTransportMode_UDP)
                ElseIf rdbTransportTCP.Checked Then
                    m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, Datastead.RTSPFilter.RTSPTransportMode_TCP)
                ElseIf rdbTransportHTTP.Checked Then
                    m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, Datastead.RTSPFilter.RTSPTransportMode_HTTP)
                ElseIf rdbTransportMulticast.Checked Then
                    m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_RTSPTransport_int, Datastead.RTSPFilter.RTSPTransportMode_Udp_Multicast)
                End If

                If chkVidSync.CheckState <> CheckState.Indeterminate Then
                    m_Graph.m_DatasteadRTSPConfigHelper.SetBool(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_Synchronized_bool, chkVidSync.Checked)
                End If

                If chkLowDelay.CheckState <> CheckState.Indeterminate Then
                    ' 0 = disabled, 1 = enabled
                    m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_LowDelay_int, Convert.ToInt32(chkLowDelay.Checked))
                End If

                If chkUseSelectedAudioDeviceAsAudioSource.Checked Then
                    If lstAudioDevices.Items.Count > 0 Then
                        If lstAudioDevices.SelectedIndex > -1 Then
                            Dim SelectedAudioDevice As String = lstAudioDevices.Items(lstAudioDevices.SelectedIndex).ToString()
                            m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_SetAudioDevice_str, SelectedAudioDevice)
                        End If
                    End If
                End If

                Dim BufferValue As Integer
                If Int32.TryParse(tbBuffering.Text, BufferValue) Then
                    Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_BufferDuration_int, BufferValue)
                    LogIfError(hr, "set buffer duration")
                End If

                Dim TimePositionValue As Integer
                If Int32.TryParse(tbTimePosition.Text, TimePositionValue) Then
                    If TimePositionValue > 0 Then
                        Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_StartTime_int, TimePositionValue)
                        LogIfError(hr, "set start time")
                    End If
                End If

                Dim FrameRate As Double
                If Double.TryParse(tbFrameRate.Text, FrameRate) Then
                    Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.SetDouble(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_FrameRateMax_double, FrameRate)
                    LogIfError(hr, "set frame rate")
                End If

                If tbUserName.Text <> String.Empty Then
                    Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_AuthUser_str, tbUserName.Text)
                    LogIfError(hr, "set user authentication")
                End If

                If tbPassword.Text <> String.Empty Then
                    Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_AuthPassword_str, tbPassword.Text)
                    LogIfError(hr, "set password authentication")
                End If

                If chkRestreamURL.Checked Then
                    Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Dest_URL_str, edtRestreamURL.Text)
                    LogIfError(hr, "set restream URL")
                End If


                If cbRecord.Checked Then
                    Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_RecordingFileName_str, tbRecordingFileName.Text)
                    LogIfError(hr, "set recording file name")

                    If m_backtimed_recording_start_in_seconds > 0 Then
                        hr = m_Graph.m_DatasteadRTSPConfigHelper.SetInt(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Source_RecordingBacktimedStartSeconds_int, m_backtimed_recording_start_in_seconds)
                        LogIfError(hr, "set backtimed recording start in seconds")
                        hr = m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_PauseRecording, "")
                        LogIfError(hr, "put the recording in paused mode")
                        ' will be resumed when needed while the graph is running by invoking Action(RTSP_Action_ResumeRecording, "")
                        MessageBox.Show(hr.ToString())
                    End If
                End If

                Dim URL As String = tbURL.Text

                m_Graph.GraphRunResult = AddressOf GraphRunResultCallback

                If OpenAsynchronously Then
                    ' Action (RTSP_Action_OpenURLAsync) opens the URL asynchrnously
                    Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_OpenURLAsync, URL)
                    LogIfError(hr, "open URL asynchronously")
                    ''' starting from here the filter connects asynchrnously to the URL and notify the graph through IMediaEventEx by sending EC_RTSPNOTIFY (EC_RTSP_PARAM1_OPENURLASYNC_SUCCEEDED, 0), see the HandleGraphEvent function below
                    If hr = 0 Then
                        AddToLog("Connecting asynchronously...")
                    End If
                Else
                    ' Action (RTSP_Action_OpenURL) opens the URL synchrnously
                    AddToLog("Connecting synchronously...")
                    Application.DoEvents()
                    ''' to update the log window
                    Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_OpenURL, URL)
                    LogIfError(hr, "open URL synchronously")
                    If hr = 0 Then
                        m_Graph.RunGraph()
                    End If
                End If
            End If
        End If
        Return True
    End Function

    Public Sub GraphRunResultCallback(ByVal Success As Boolean)
        AddToLog("running")
        btnPauseRecording.Enabled = cbRecord.Checked
        btnResumeRecording.Enabled = cbRecord.Checked
        btnGenerateNewFile.Enabled = cbRecord.Checked
    End Sub

    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        m_Graph.ResizeVideoWindow()
    End Sub

    Private Sub btnInfoLimitations_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnInfoLimitations.Click
        Const info As String = "- LIMITATIONS OF THE EVALUATION VERSION:" & vbCr & vbLf & "- several filter instances can be run concurrently in the same application, each instance will stop randomly and independently after a few minutes to a few hours." & vbCr & vbLf & "- Once one of the instances has stopped, no other instance can be (re)started until the application is restarted."
        MessageBox.Show(info)

    End Sub

    Private Sub btnGenerateNewFile_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnGenerateNewFile.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            ' if you pass string.Empty as 2nd parameter the current file being recorded will be overwritten
            Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_RecordToNewFileNow, tbRecordingFileName.Text)
            LogIfError(hr, "record to new file failed")
        End If
    End Sub

    Private Sub btnOpenSync_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnOpenSync.Click
        m_Graph.CloseInterfaces()
        OpenURL(False)
        ' starting from here the filter connects asynchrously, 
        ' then when the connection completes the filter sends
        ' an EC_RTSP_NOTIFY event to the graph event handler (HandleGraphEvent) 
        ' that invokes CompleteGraph()
        ' >>>>> SEE HandleGraphEvent() and CompleteGraph() <<<<<
    End Sub

    Private Sub btnOpenAsync_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnOpenAsync.Click
        m_Graph.CloseInterfaces()
        OpenURL(True)
        ' starting from here the filter is connected and the graph running
    End Sub

    Private Sub btnStop_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnStop.Click
        m_Graph.CloseInterfaces()
        AddToLog("stopped.")
    End Sub

    Private Sub btnPause_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnPause.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_PauseURL, "")
        End If
    End Sub

    Private Sub btnResume_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnResume.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_ResumeURL, "")
        End If
    End Sub

    Private Sub btnBufferInfo_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnBufferInfo.Click
        MessageBox.Show("Specify explicitely 0 to disable the default buffering. If the video freezes periodically, increase the value, e.g. between 200 and 1000 ms, or disable the video sync if not required")
    End Sub

    Private Sub btnRestreamInfo_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnRestreamInfo.Click
        MessageBox.Show("destination IP:port URL to which the video will be re-streamed:" & vbLf & vbLf + "- UDP unicast, e.g.:" & vbLf + "udp://192.168.2:45:10000" & vbLf & vbLf + "- UDP multicast, e.g.:" & vbLf + "udp://239.255.0.10:11000" & vbLf & vbLf + "- RTSP server, e.g.:" & vbLf + "rtspsrv://192.168.0.22:12000")
    End Sub

    Private Sub btnCaptureToJPG_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCaptureToJPG.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "snapshot.jpg")
        End If
    End Sub

    Private Sub btnCaptureToPNG_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCaptureToPNG.Click
        If IntPtr.Size > 4 Then
            MessageBox.Show(".png format not yet supported in x64 build, this will be fixed in future versions, for the moment use the x86 build or choose another format")
        Else
            If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
                m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "snapshot.png")
            End If
        End If
    End Sub

    Private Sub btnCaptureToTIFF_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCaptureToTIFF.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "snapshot.tiff")
        End If

    End Sub

    Private Sub btnCaptureToBMP_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCaptureToBMP.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "snapshot.bmp")
        End If
    End Sub

    Private Sub btnCaptureToMemoryBitmap_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCaptureToMemoryBitmap.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            ' "HBITMAP" is a reserved keyword. For the corresponding sample code search for "m_PictureFromMemoryBitmap" in this project
            m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_CaptureFrame, "HBITMAP")
        End If
    End Sub

    Private Sub btnGetAudioDevicesList_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnGetAudioDevicesList.Click
        lstAudioDevices.Items.AddRange(m_Graph.GetAudioDevicesAsItems())
    End Sub

    Private Sub btnGenerateNewFileInfo_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnGenerateNewFileInfo.Click
        MessageBox.Show("To pause the recording, enter a file name with the same extension, but ""nul"" as name and no file path, e.g. ""nul.mp4"".  If the new file name is identical the previous recording is overwritten")
    End Sub

    Private Sub txtTextOverlay_TextChanged(ByVal sender As Object, ByVal e As EventArgs) Handles txtTextOverlay.TextChanged
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            ' we can update the overlay in real time
            m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureTextOverlay_str, DirectCast(sender, TextBox).Text)
        End If
        Warn_TextOverlay(sender, e)
    End Sub

    Private Sub txtBrightHueSat_TextChanged(ByVal sender As Object, ByVal e As EventArgs) Handles txtBrightHueSat.TextChanged
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            ' we can update the hue, brightness and saturation in real time
            m_Graph.m_DatasteadRTSPConfigHelper.SetStr(Datastead.RTSPFilter.RTSPConfigParam.RTSP_VideoStream_ConfigureHueBrightSat_str, txtBrightHueSat.Text)
        End If
        Warn_BrightHueSat(sender, e)
    End Sub

    Private m_TextOverlayWarningNotified As Boolean = False
    Private Sub Warn_TextOverlay(ByVal sender As Object, ByVal e As EventArgs)
        If Not m_TextOverlayWarningNotified Then
            m_TextOverlayWarningNotified = True
            AddToLog("Warning: each overlay must be set BEFORE LOADING the URL. To prevent displaying an overlay immediately, set an empty text string, then update the text string when needed, while the filter is running.")
            AddToLog("Warning: this edit field is for quick and easier demonstration purpose only. An incorrect syntax (e.g. wrong color name) may crash the filter, especially when updating it in real time.")
        End If
    End Sub
    Private m_BrightHueSatWarningNotified As Boolean = False
    Private Sub Warn_BrightHueSat(ByVal sender As Object, ByVal e As EventArgs)
        If Not m_BrightHueSatWarningNotified Then
            m_BrightHueSatWarningNotified = True
            AddToLog("brightness: in the -10..10 range (default 0)")
            AddToLog("hue: in degrees (default 0)")
            AddToLog("saturation: in the -10..10 range (default 1)")
            AddToLog("Warning: the brightness/hue/saturation setting must be set BEFORE LOADING the URL. To prevent it to be applied immediately, set the default value(s) (b=0,h=0,s=1), then update them when needed while the filter is running.")
        End If
    End Sub

    Private Sub chkVideoStreamEnabled_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles chkVideoStreamEnabled.CheckedChanged
        m_Graph.m_VideoStreamEnabled = chkVideoStreamEnabled.Checked
    End Sub

    Private Sub chkAudioStreamEnabled_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles chkAudioStreamEnabled.CheckedChanged
        m_Graph.m_AudioStreamEnabled = chkAudioStreamEnabled.Checked
    End Sub

    Private Sub chkVideoStreamVisible_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles chkVideoStreamVisible.CheckedChanged
        m_Graph.m_VideoStreamVisible = chkVideoStreamVisible.Checked
    End Sub

    Private Sub chkAudioStreamRendered_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles chkAudioStreamRendered.CheckedChanged
        m_Graph.m_AudioStreamRendered = chkAudioStreamRendered.Checked
    End Sub

    Private Sub rdgVideoRendererDefault_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles rdgVideoRendererDefault.CheckedChanged
        m_Graph.m_VideoRendererToUse = DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_Default
    End Sub

    Private Sub rdgDatasteadVideoRenderer_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles rdgDatasteadVideoRenderer.CheckedChanged
        m_Graph.m_VideoRendererToUse = DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_DatasteadRenderer
    End Sub

    Private Sub rdgStandardVideoRenderer_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles rdgStandardVideoRenderer.CheckedChanged
        m_Graph.m_VideoRendererToUse = DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_StandardRenderer
    End Sub

    Private Sub rdgVideoRendererVMR7_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles rdgVideoRendererVMR7.CheckedChanged
        m_Graph.m_VideoRendererToUse = DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_VMR7
    End Sub

    Private Sub rdgVideoRendererVMR9_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles rdgVideoRendererVMR9.CheckedChanged
        m_Graph.m_VideoRendererToUse = DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_VMR9
    End Sub

    Private Sub rdgNullRenderer_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles rdgNullRenderer.CheckedChanged
        m_Graph.m_VideoRendererToUse = DatasteadDirectShow.DatasteadDirectShowGraph_WithEventsMsg.VideoRendererToUse.vr_NullRenderer
    End Sub

    Private Sub btnPauseRecording_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnPauseRecording.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            ' if you pass string.Empty as 2nd parameter the current file being recorded will be overwritten
            Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_PauseRecording, "")
            LogIfError(hr, "pause recording failed")
        End If

    End Sub
    Private Sub btnResumeRecording_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnResumeRecording.Click
        If m_Graph.m_DatasteadRTSPSourceConfig IsNot Nothing Then
            ' if you pass string.Empty as 2nd parameter the current file being recorded will be overwritten
            Dim hr As Integer = m_Graph.m_DatasteadRTSPConfigHelper.Action(Datastead.RTSPFilter.RTSPConfigParam.RTSP_Action_ResumeRecording, "")
            LogIfError(hr, "pause recording failed")
        End If
    End Sub

    Private Sub btnBacktimedStartInfo_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnBacktimedStartInfo.Click
        MessageBox.Show("It is possible to specify a number of seconds that must be included at the beginning of the recording, BEFORE RecordToNewFileNow or ResumeRecording has been invoked." & vbLf & vbLf & "To use this feature the recording must be started in ""paused"" mode by invoking Action(RTSP_Action_PauseRecording) before starting the graph. " & vbLf & vbLf & "Then to begin writing to the file invoke Action(RTSP_Action_ResumeRecording) or Action(RTSP_Action_RecordToNewFileNow, newfilename.mp4)." & vbLf & vbLf & "This will include in the video clip the specified number of seconds of video just before the user decided to start the recording.")

    End Sub
    Private Sub cbRecord_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles cbRecord.CheckedChanged
        labelBacktimedSeconds.Enabled = cbRecord.Checked
        txtBacktimedStartSeconds.Enabled = cbRecord.Checked
        btnBacktimedRecSecondsOK.Enabled = cbRecord.Checked

    End Sub

    Private Sub btnBacktimedRecSecondsOK_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnBacktimedRecSecondsOK.Click
        m_backtimed_recording_start_in_seconds = 0
        If Integer.TryParse(txtBacktimedStartSeconds.Text, m_backtimed_recording_start_in_seconds) Then
            If m_backtimed_recording_start_in_seconds > 0 Then
                MessageBox.Show("the graph will start without writing to the file immediately, when you want to begin writing to the file, press ""Resume Recording""")
            End If
        End If
    End Sub
End Class
